@@ -221,6 +221,32 @@ Array::append_dim(int size, string name)
     update_length(size);
 }
 
+/** Creates a new OUTER dimension (slowest varying in rowmajor)
+ * for the array by prepending rather than appending it.
+ * @param size cardinality of the new dimension
+ * @param name  optional name for the new dimension
+ */
+void
+Array::prepend_dim(int size, const string& name/* = "" */)
+{
+  dimension d;
+
+  // This is invariant
+  d.size = size;
+  d.name = www2id(name);
+
+  // this information changes with each constraint expression
+  d.start = 0;
+  d.stop = size - 1;
+  d.stride = 1;
+  d.c_size = size;
+
+  // Shifts the whole array, but it's tiny in general
+  _shape.insert(_shape.begin(), d);
+
+  update_length(size); // the number is ignored...
+}
+
 /** Resets the dimension constraint information so that the entire
     array is selected.
 
@@ -471,7 +497,8 @@ Array::dimension_name(Dim_iter i)
                            "*This* array has no dimensions.");
     return (*i).name;
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 /** Prints a declaration for the Array.  This is what appears in a
     DDS.  If the Array is constrained, the declaration will reflect
     the size of the Array once the constraint is applied.
@@ -516,7 +543,8 @@ Array::print_decl(FILE *out, string space, bool print_semi,
         fprintf(out, ";\n") ;
     }
 }
-//#endif
+#endif
+
 /** Prints a declaration for the Array.  This is what appears in a
     DDS.  If the Array is constrained, the declaration will reflect
     the size of the Array once the constraint is applied.
@@ -561,31 +589,31 @@ Array::print_decl(ostream &out, string space, bool print_semi,
 	out << ";\n" ;
     }
 }
-//#if FILE_METHODS
+#if FILE_METHODS
 void
 Array::print_xml(FILE *out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Array");
 }
-//#endif
+#endif
 void
 Array::print_xml(ostream &out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Array");
 }
-//#if FILE_METHODS
+#if FILE_METHODS
 void
 Array::print_as_map_xml(FILE *out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Map");
 }
-//#endif
+#endif
 void
 Array::print_as_map_xml(ostream &out, string space, bool constrained)
 {
     print_xml_core(out, space, constrained, "Map");
 }
-//#if FILE_METHODS
+#if FILE_METHODS
 class PrintArrayDim : public unary_function<Array::dimension&, void>
 {
     FILE *d_out;
@@ -632,7 +660,8 @@ Array::print_xml_core(FILE *out, string space, bool constrained, string tag)
 
     fprintf(out, "%s</%s>\n", space.c_str(), tag.c_str());
 }
-//#endif
+#endif
+
 class PrintArrayDimStrm : public unary_function<Array::dimension&, void>
 {
     ostream &d_out;
@@ -678,7 +707,8 @@ Array::print_xml_core(ostream &out, string space, bool constrained, string tag)
 
     out << space << "</" << tag << ">\n" ;
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 /** Prints the values in ASCII of the entire (constrained) array. This method
     Attempts to make an aesthetically pleasing display. However, it is
     primarily intended for debugging purposes.
@@ -722,7 +752,8 @@ Array::print_array(FILE *out, unsigned int index, unsigned int dims,
         return index;
     }
 }
-//#endif
+#endif
+
 /** Prints the values in ASCII of the entire (constrained) array. This method
     Attempts to make an anesthetically pleasing display. However, it is
     primarily intended for debugging purposes.
@@ -766,7 +797,8 @@ Array::print_array(ostream &out, unsigned int index, unsigned int dims,
         return index;
     }
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 void
 Array::print_val(FILE *out, string space, bool print_decl_p)
 {
@@ -794,7 +826,8 @@ Array::print_val(FILE *out, string space, bool print_decl_p)
         fprintf(out, ";\n") ;
     }
 }
-//#endif
+#endif
+
 void
 Array::print_val(ostream &out, string space, bool print_decl_p)
 {
