@@ -51,12 +51,12 @@ private:
                 : name(n), b_func(f), bt_func(0), p_func(0)
         {}
         function(const string &n, const btp_func f)
-                : name(n), b_func(0), bt_func(f), p_func(0)
+                : name(n), bt_func(f), p_func(0)
         {}
         function(const string &n, const proj_func f)
-                : name(n), b_func(0), bt_func(0), p_func(f)
+                : name(n), bt_func(0), p_func(f)
         {}
-        function(): name(""), b_func(0), bt_func(0), p_func(0)
+        function(): name(""), bt_func(0), p_func(0)
         {}
     };
 
@@ -111,7 +111,7 @@ public:
 
     Clause_iter clause_begin();
     Clause_iter clause_end();
-    bool clause_value(Clause_iter &i, DDS &dds, const string &dataset);
+    bool clause_value(Clause_iter &i, DDS &dds/*, const string &dataset***/);
 
     void parse_constraint(const string &constraint, DDS &dds);
     void append_constant(BaseType *btp);
