@@ -34,7 +34,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: DDS.cc 20697 2009-04-08 17:14:10Z jimg $"
+    {"$Id: DDS.cc 22560 2010-04-21 21:22:00Z jimg $"
     };
 
 #include <cstdio>
@@ -67,10 +67,25 @@ static char rcsid[] not_used =
 #include "parser.h"
 #include "debug.h"
 #include "util.h"
+
+#include "Byte.h"
+#include "Int16.h"
+#include "UInt16.h"
+#include "Int32.h"
+#include "UInt32.h"
+#include "Float32.h"
+#include "Float64.h"
+#include "Str.h"
+#include "Url.h"
+#include "Array.h"
+#include "Structure.h"
+#include "Sequence.h"
+#include "Grid.h"
+
 #include "escaping.h"
 
 const string c_default_dap20_schema_location = "http://xml.opendap.org/dap/dap2.xsd";
-const string c_default_dap32_schema_location = "http://xml.opendap.org/dap/dap/3.2.xsd";
+const string c_default_dap32_schema_location = "http://xml.opendap.org/dap/dap3.2.xsd";
 
 const string c_dap20_namespace = "http://xml.opendap.org/ns/DAP2";
 const string c_dap32_namespace = "http://xml.opendap.org/ns/DAP/3.2#";
@@ -96,29 +111,15 @@ DDS::duplicate(const DDS &dds)
 {
     DBG(cerr << "Entering DDS::duplicate... " <<endl);
     name = dds.name;
+    d_filename = dds.d_filename;
+    d_container_name = dds.d_container_name;
+    d_timeout = dds.d_timeout;
+    d_attr = dds.d_attr;
+
     d_factory = dds.d_factory;
     d_container = dds.d_container;
-
-    d_client_dap_major = dds.d_client_dap_major;
-    d_client_dap_minor = dds.d_client_dap_minor;
-
-
-#if 0
-    //fields to copy
-    string _filename;       // File name (or other OS identifier) for
-    string _container_name; // name of container structure
-    int d_protocol_major;       // The protocol major version number
-    int d_protocol_minor;       // ... and minor version number
-
-    // These hold the major and minor versions of DAP that the client send in
-    // the XDAP-Accept header. If the header is not sent, these default to 2.0
-    int d_client_dap_major;
-    int d_client_dap_minor;
-
-    AttrTable d_attr;           // Global attributes.
-    int d_timeout;              // alarm time in seconds. If greater than
-
-#endif
+    d_dap_major = dds.d_dap_major;
+    d_dap_minor = dds.d_dap_minor;
 
     DDS &dds_tmp = const_cast<DDS &>(dds);
 
@@ -141,11 +142,11 @@ DDS::duplicate(const DDS &dds)
 DDS::DDS(BaseTypeFactory *factory, const string &n)
 
         : d_factory(factory), name(n), d_container(0), d_dap_major(2),
-        d_dap_minor(0), d_client_dap_major(2), d_client_dap_minor(0),
+        d_dap_minor(0),
         d_request_xml_base(""), d_timeout(0)
 {
     DBG(cerr << "Building a DDS with client major/minor: "
-            << d_client_dap_major << "." << d_client_dap_minor << endl);
+            << d_dap_major << "." << d_dap_minor << endl);
 }
 
 /** The DDS copy constructor. */
@@ -178,6 +179,7 @@ DDS::operator=(const DDS &rhs)
     return *this;
 }
 
+#if 0
 /** @brief Look for the parent of an HDF4 dimension attribute
 
     If this attribute container's name ends in the '_dim_?' suffix, look
@@ -225,7 +227,7 @@ DDS::find_matching_container(AttrTable::entry *source, BaseType **dest_variable)
 {
     // The attribute entry 'source' must be a container
     if (source->type != Attr_container)
-        throw InternalErr(__FILE__, __LINE__, "DDS::find_matching_container");
+        throw InternalErr(__FILE__, __LINE__, "DDS::find_matching_container; expected 'source' to be a container.");
 
     // Use the name of the attribute container 'source' to figure out where
     // to put its contents.
@@ -243,7 +245,7 @@ DDS::find_matching_container(AttrTable::entry *source, BaseType **dest_variable)
             *dest_variable = btp;
             return &btp->get_attr_table();
         }
-        else { // must ba a plain Array
+        else { // must be a plain Array
             string::size_type i = source->name.find("_dim_");
             string ext = source->name.substr(i + 1);
             *dest_variable = btp;
@@ -293,13 +295,14 @@ DDS::transfer_attributes(DAS *das)
     // exist in the DAS, then throw an exception
     if( d_container )
     {
-	if( das->container_name() != _container_name )
+	if( das->container_name() != d_container_name )
 	{
 	    string err = (string)"Error transferring attributes: "
 			 + "working on container in dds, but not das" ;
 	    throw InternalErr(__FILE__, __LINE__, err ) ;
 	}
     }
+
     AttrTable *top_level = das->get_top_level_attributes() ;
 
     // foreach container at the outer level
@@ -320,25 +323,15 @@ DDS::transfer_attributes(DAS *das)
             DBG(cerr << "Working on the '" << (*source_p)->name << "' attribute"
                 << endl);
 
-            // If this is container, we must have a container (this one) within
-            // a container (the 'source'). Look and see if the variable is a
-            // Constructor. If so, pass that container into
+            // If this is an attribute container, we must have a container
+            // (this one) within a container (the 'source'). Look and see if
+            // the variable is a Constructor. If so, pass that container into
             // Constructor::transfer_attributes()
             if ((*source_p)->type == Attr_container) {
                 if (dest_variable && dest_variable->is_constructor_type()) {
                     dynamic_cast<Constructor&>(*dest_variable).transfer_attributes(*source_p);
                 }
                 else {
-#if 0
-                    // Trick:
-                    DBG(cerr << "Is current source aliased: "
-                            << (*source_p)->is_alias << endl);
-                    AttrTable &local = *(*source_p)->attributes;
-                    for (AttrTable::Attr_iter i = local.attr_begin();
-                         i != local.attr_end(); ++i) {
-                        cerr << "... or one of its entries: " << (*i)->is_alias << endl;
-                    }
-#endif
                     dest->append_container(new AttrTable(*(*source_p)->attributes),
                                            (*source_p)->name);
                 }
@@ -355,6 +348,88 @@ DDS::transfer_attributes(DAS *das)
         ++das_i;
     }
 }
+#endif
+
+/**
+ * This is the main method used to transfer attributes from a DAS object into a
+ * DDS. This uses the BaseType::transfer_attributes() method and the various
+ * implementations found here (in the constructors classes) and in handlers.
+ *
+ * This method uses a deep copy to transfer the attributes, to it's safe to
+ * delete the source DAS object passed to this method once it's done.
+ *
+ * @note To accommodate oddly built DAS objects produced by various handlers,
+ * specialize the methods there.
+ *
+ * @param das Transfer (copy) attributes from this DAS object.
+ */
+void
+DDS::transfer_attributes(DAS *das)
+{
+    // If there is a container set in the DDS then get the container from
+    // the DAS. If they are not the same container, then throw an exception
+    // (should be working on the same container). If the container does not
+    // exist in the DAS, then throw an exception
+    if( d_container )
+    {
+	if( das->container_name() != d_container_name )
+	    throw InternalErr(__FILE__, __LINE__, "Error transferring attributes: working on a container in dds, but not das" ) ;
+    }
+
+    // Give each variable a chance to claim its attributes.
+    AttrTable *top_level = das->get_top_level_attributes() ;
+
+    Vars_iter var = var_begin();
+    while (var != var_end()) {
+	(*var)->transfer_attributes(top_level);
+	var++;
+    }
+
+    // Now we transfer all of the attributes still marked as global to the
+    // global container in the DDS.
+
+    AttrTable::Attr_iter at_cont_p = top_level->attr_begin();
+    while (at_cont_p != top_level->attr_end()) {
+	// In truth, all of the top level attributes should be containers, but
+	// this test handles the abnormal case where somehow someone makes a
+	// top level attribute that is not a container by silently dropping it.
+	if ((*at_cont_p)->type == Attr_container
+		&& (*at_cont_p)->attributes->is_global_attribute()) {
+	    DBG(cerr << (*at_cont_p)->name << " is a global attribute." << endl);
+	    // copy the source container so that the DAS passed in can be
+	    // deleted after calling htis method.
+	    AttrTable *at = new AttrTable(*(*at_cont_p)->attributes);
+	    d_attr.append_container(at, at->get_name());
+	}
+
+	at_cont_p++;
+    }
+}
+
+#if 0
+    // cruft from the above method
+
+	    AttrTable *dest = d_attr.find_container(at->get_name());
+	    if (!dest) {
+		cerr << "making a new sub containter for it" << endl;
+		// If there's currently no top level container with this
+		//container's name (the typical case) make one.
+		dest = new AttrTable(); // Make a new global table if needed
+		d_attr.append_container(dest, at->get_name());
+	    }
+
+	    cerr << "now copying its contents to the new container" << endl;
+	    // Now copy all of the global attribute's stuff into the matching
+	    // container in the DDS.
+	    AttrTable::Attr_iter at_p = at->attr_begin();
+	    while (at_p != at->attr_end()) {
+		if (at->get_attr_type(at_p) == Attr_container)
+		    dest->append_container(at->get_attr_table(at_p), at->get_name(at_p));
+		else
+		    dest->append_attr(at->get_name(at_p), at->get_type(at_p), at->get_attr_vector(at_p));
+		at_p++;
+	    }
+#endif
 
 /** Get and set the dataset's name.  This is the name of the dataset
     itself, and is not to be confused with the name of the file or
@@ -399,14 +474,14 @@ DDS::get_attr_table()
 string
 DDS::filename()
 {
-    return _filename;
+    return d_filename;
 }
 
 /** Set the dataset's filename. */
 void
 DDS::filename(const string &fn)
 {
-    _filename = fn;
+    d_filename = fn;
 }
 //@}
 
@@ -435,34 +510,6 @@ DDS::set_dap_version(const string &version_string)
     set_dap_minor(minor);
 }
 
-/** Given a version string passed to a server from a client in the XDAP-Accept
-    MIME header, parse that string and record the major and minor protocol
-    version numbers. This method differs from set_dap_version() in that it is
-    storing the version that the client would _like_ the server to use. The
-    actual protocol version to which this DDS/DDX conforms is found using the
-    get_dap_major() and get_dap_minor() methods.
-    @param version_string
- */
-void
-DDS::set_client_dap_version(const string &version_string)
-{
-    istringstream iss(version_string);
-
-    int major = -1, minor = -1;
-    char dot;
-    iss >> major;
-    iss >> dot;
-    iss >> minor;
-
-    DBG(cerr << "Major: " << major << ", dot: " << dot <<", Minor: " << minor << endl);
-
-    if (major == -1 || minor == -1)
-        throw Error("Could not parse the client dap (XDAP-Accept header) value");
-
-    set_client_dap_major(major);
-    set_client_dap_minor(minor);
-}
-
 /** Get and set the current container. If there are multiple files being
     used to build this DDS, using a container will set a virtual structure
     for the current container.
@@ -475,7 +522,7 @@ DDS::set_client_dap_version(const string &version_string)
 string
 DDS::container_name()
 {
-    return _container_name;
+    return d_container_name;
 }
 
 /** Set the current container name and get or create a structure for that
@@ -503,7 +550,7 @@ DDS::container_name(const string &cn)
 	    d_container = dynamic_cast<Structure *>( var( cn ) ) ;
 	}
     }
-    _container_name = cn;
+    d_container_name = cn;
 
 }
 
@@ -534,7 +581,11 @@ DDS::add_var(BaseType *bt)
     DBG2(cerr << "In DDS::add_var(), btp's address is: " << btp << endl);
     if( d_container )
     {
+        // Mem leak fix [mjohnson nov 2009]
+        // Structure::add_var() creates ANOTHER copy.
 	d_container->add_var( bt ) ;
+	// So we need to delete btp or else it leaks
+	delete btp; btp = 0;
     }
     else
     {
@@ -878,7 +929,8 @@ DDS::parse(FILE *in)
             throw *arg.error();
     }
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 /** @brief Print the entire DDS to the specified file. */
 void
 DDS::print(FILE *out)
@@ -893,7 +945,8 @@ DDS::print(FILE *out)
 
     return ;
 }
-//#endif
+#endif
+
 /** @brief Print the entire DDS to the specified ostream. */
 void
 DDS::print(ostream &out)
@@ -908,7 +961,8 @@ DDS::print(ostream &out)
 
     return ;
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 /** @brief Print a constrained DDS to the specified file.
 
     Print those parts (variables) of the DDS structure to OS that
@@ -935,7 +989,8 @@ DDS::print_constrained(FILE *out)
 
     return;
 }
-//#endif
+#endif
+
 /** @brief Print a constrained DDS to the specified ostream.
 
     Print those parts (variables) of the DDS structure to OS that
@@ -962,7 +1017,8 @@ DDS::print_constrained(ostream &out)
 
     return;
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 class VariablePrintXML : public unary_function<BaseType *, void>
 {
     FILE *d_out;
@@ -976,8 +1032,7 @@ public:
         bt->print_xml(d_out, "    ", d_constrained);
     }
 };
-//#endif
-//#if FILE_METHODS
+
 /** Print an XML representation of this DDS. This method is used to generate
     the part of the DDX response. The \c Dataset tag is \e not written by
     this code. The caller of this method must handle writing that and
@@ -989,7 +1044,7 @@ public:
     expression.
     @param blob The dataBLOB href. */
 void
-DDS::print_xml(FILE *out, bool constrained, const string &)
+DDS::print_xml(FILE *out, bool constrained, const string &blob)
 {
     fprintf(out, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 
@@ -998,12 +1053,12 @@ DDS::print_xml(FILE *out, bool constrained, const string &)
     fprintf(out, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
 
     fprintf(out,"method=\"FILE*\"\n");
-    fprintf(out, "dap_major=\"%d\"\n", get_client_dap_major());
-    fprintf(out, "dap_minor=\"%d\"\n", get_client_dap_minor());
+    fprintf(out, "dap_major=\"%d\"\n", get_dap_major());
+    fprintf(out, "dap_minor=\"%d\"\n", get_dap_minor());
 
     // Are we responding to a 3.2 or 2.0 client? We will have to improve on
     // this at some point... jhrg
-    if (get_client_dap_major() == 3 && get_client_dap_minor() == 2) {
+    if (get_dap_major() == 3 && get_dap_minor() == 2) {
     fprintf(out, "xmlns=\"%s\"\n", c_dap32_namespace.c_str());
 
     fprintf(out, "xsi:schemaLocation=\"%s  %s\">\n\n",
@@ -1026,13 +1081,19 @@ DDS::print_xml(FILE *out, bool constrained, const string &)
 
     // Only print this for the 2.0, 3.0 and 3.1 versions - which are essentially
     // the same. jhrg
-    if (get_client_dap_major() == 2 && get_client_dap_minor() == 0) {
+    if (get_dap_major() == 2 && get_dap_minor() == 0) {
         fprintf(out, "    <dataBLOB href=\"\"/>\n");
     }
+    else if (!blob.empty()
+	     && (get_dap_major() == 3 && get_dap_minor() >= 2)
+	     || get_dap_major() >= 4) {
+	fprintf(out, "    <blob href=\"cid:%s\"/>\n", blob.c_str());
+    }
+
 
     fprintf(out, "</Dataset>\n");
 }
-//#endif
+#endif
 
 class VariablePrintXMLStrm : public unary_function<BaseType *, void>
 {
@@ -1059,7 +1120,7 @@ public:
     expression.
     @param blob The dataBLOB href. */
 void
-DDS::print_xml(ostream &out, bool constrained, const string &)
+DDS::print_xml(ostream &out, bool constrained, const string &blob)
 {
     out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" ;
 
@@ -1069,7 +1130,7 @@ DDS::print_xml(ostream &out, bool constrained, const string &)
 
     // Are we responding to a 3.2 or 2.0 client? We will have to improve on
     // this at some point... jhrg
-    if (get_client_dap_major() == 3 && get_client_dap_minor() == 2) {
+    if (get_dap_major() == 3 && get_dap_minor() == 2) {
         out << "xsi:schemaLocation=\"" << c_dap32_namespace
             << "  " << c_default_dap32_schema_location << "\"\n" ;
 
@@ -1079,12 +1140,13 @@ DDS::print_xml(ostream &out, bool constrained, const string &)
         out << "xmlns=\"" << c_dap32_namespace << "\"\n" ;
         out << "xmlns:dap=\"" << c_dap32_namespace << "\"\n" ;
 
-        out << "dap_version=\"" << get_client_dap_major() << "."
-            << get_client_dap_minor() << "\"\n";
+        out << "dapVersion=\"" << get_dap_major() << "."
+            << get_dap_minor() << "\"";
 
         if (!get_request_xml_base().empty()) {
+            out << "\n";
             out << "xmlns:xml=\"" << c_xml_namespace << "\"\n";
-            out << "xml:base=\"" << get_request_xml_base() << "\"\n";
+            out << "xml:base=\"" << get_request_xml_base() << "\"";
         }
 
         // Close the Dataset element
@@ -1105,10 +1167,17 @@ DDS::print_xml(ostream &out, bool constrained, const string &)
     out << "\n" ;
 
     // Only print this for the 2.0, 3.0 and 3.1 versions - which are essentially
-    // the same. jhrg
-    if (get_client_dap_major() == 2 && get_client_dap_minor() == 0) {
+    // the same.
+    // For DAP 3.2 and greater, use the new syntax and value. The 'blob' is
+    // actually the CID of the MIME part that holds the data.
+    if (get_dap_major() == 2 && get_dap_minor() == 0) {
         out << "    <dataBLOB href=\"\"/>\n" ;
     }
+    else if (!blob.empty()
+	     && (get_dap_major() == 3 && get_dap_minor() >= 2)
+	     || get_dap_major() >= 4) {
+	out << "    <blob href=\"cid:" << blob << "\"/>\n";
+    }
 
     out << "</Dataset>\n" ;
 }
@@ -1239,7 +1308,7 @@ DDS::dump(ostream &strm) const
     << (void *)this << ")" << endl ;
     DapIndent::Indent() ;
     strm << DapIndent::LMarg << "name: " << name << endl ;
-    strm << DapIndent::LMarg << "filename: " << _filename << endl ;
+    strm << DapIndent::LMarg << "filename: " << d_filename << endl ;
     strm << DapIndent::LMarg << "protocol major: " << d_dap_major << endl;
     strm << DapIndent::LMarg << "protocol minor: " << d_dap_minor << endl;
     strm << DapIndent::LMarg << "factory: " << (void *)d_factory << endl ;
