@@ -37,7 +37,7 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    {"$Id: DODSFilter.cc 19948 2008-12-03 23:51:12Z jimg $"
+    {"$Id: DODSFilter.cc 21701 2009-11-06 00:35:54Z jimg $"
     };
 
 #include <signal.h>
@@ -55,26 +55,36 @@ static char rcsid[] not_used =
 #include <string>
 #include <algorithm>
 #include <cstdlib>
+#include <cstring>
+
+#include <uuid/uuid.h>	// used to build CID header value for data ddx
 
 #include <GetOpt.h>
 
 #include "DAS.h"
 #include "DDS.h"
 #include "debug.h"
-#include "cgi_util.h"
+#include "mime_util.h"
 #include "Ancillary.h"
 #include "util.h"
 #include "escaping.h"
 #include "DODSFilter.h"
+#if FILE_METHODS
 #include "XDRFileMarshaller.h"
+#endif
 #include "XDRStreamMarshaller.h"
 #include "InternalErr.h"
+
 #ifndef WIN32
 #include "SignalHandler.h"
 #include "EventHandler.h"
 #include "AlarmHandler.h"
 #endif
 
+#define CRLF "\r\n"             // Change here, expr-test.cc and DODSFilter.cc
+
+//#undef FILE_METHODS
+
 using namespace std;
 
 namespace libdap {
@@ -95,17 +105,6 @@ const string usage =
     -t <seconds>: Timeout the handler after <seconds>.\n\
     -h: This message.";
 
-#if 0
-// Removed the call to waitpid in send_data() because calling fflush
-// addresses the problem wait() was supposed to solve and calling wait() is
-// the root of ticket #335. jhrg 3/10/06
-#ifdef WIN32
-#define WAITPID(pid) while(_cwait(NULL, pid, NULL) > 0)
-#else
-#define WAITPID(pid) while(waitpid(pid, 0, 0) > 0)
-#endif
-#endif
-
 /** Create an instance of DODSFilter using the command line
 arguments passed by the CGI (or other) program.  The default
 constructor is private; this and the copy constructor (which is
@@ -432,6 +431,10 @@ void DODSFilter::set_response(const string &r)
 	d_response = DDX_Response;
 	d_action = "ddx" ;
     }
+    else if (r == "DataDDX" || r == "dataddx") {
+	d_response = DataDDX_Response;
+	d_action = "dataddx" ;
+    }
     else if (r == "Version") {
 	d_response = Version_Response;
 	d_action = "version" ;
@@ -606,7 +609,7 @@ DODSFilter::get_timeout() const
     return d_timeout;
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 /** Use values of this instance to establish a timeout alarm for the server.
     If the timeout value is zero, do nothing.
 
@@ -630,7 +633,7 @@ DODSFilter::establish_timeout(FILE *stream) const
     }
 #endif
 }
-//#endif
+#endif
 
 // FIXME
 void
@@ -646,41 +649,6 @@ DODSFilter::establish_timeout(ostream &stream) const
 #endif
 }
 
-
-/** Read the ancillary DAS information and merge it into the input
-    DAS object.
-
-    @brief Test if ancillary data must be read.
-    @param das A DAS object that will be augmented with the
-    ancillary data attributes.
-    @param anc_location The directory in which the external DAS file resides.
-    @return void
-    @see DAS */
-void
-DODSFilter::read_ancillary_das(DAS &das, const string &anc_location) const
-{
-    Ancillary::read_ancillary_das( das, d_dataset,
-			       (anc_location == "") ? d_anc_dir : anc_location,
-			       d_anc_file);
-}
-
-/** Read the ancillary DDS information and merge it into the input
-    DDS object.
-
-    @brief Test if ancillary data must be read.
-    @param dds A DDS object that will be augmented with the
-    ancillary data properties.
-    @param anc_location The directory in which the external DAS file resides.
-    @return void
-    @see DDS */
-void
-DODSFilter::read_ancillary_dds(DDS &dds, const string &anc_location) const
-{
-    Ancillary::read_ancillary_dds( dds, d_dataset,
-			       (anc_location == "") ? d_anc_dir : anc_location,
-			       d_anc_file);
-}
-
 static const char *emessage = "DODS internal server error; usage error. Please report this to the dataset maintainer, or to the opendap-tech@opendap.org mailing list.";
 
 /** This message is printed when the filter program is incorrectly
@@ -712,7 +680,7 @@ DODSFilter::send_version_info() const
     do_version(d_cgi_ver, get_dataset_version());
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 /** This function formats and prints an ASCII representation of a
     DAS on stdout.  This has the effect of sending the DAS object
     back to the client program.
@@ -741,7 +709,8 @@ DODSFilter::send_das(FILE *out, DAS &das, const string &anc_location,
     }
     fflush(out) ;
 }
-//#endif
+#endif
+
 /** This function formats and prints an ASCII representation of a
     DAS on stdout.  This has the effect of sending the DAS object
     back to the client program.
@@ -778,7 +747,7 @@ DODSFilter::send_das(DAS &das, const string &anc_location,
     send_das(cout, das, anc_location, with_mime_headers);
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 /** This function formats and prints an ASCII representation of a
     DDS on stdout.  When called by a CGI program, this has the
     effect of sending a DDS object back to the client
@@ -825,7 +794,8 @@ DODSFilter::send_dds(FILE *out, DDS &dds, ConstraintEvaluator &eval,
 
     fflush(out) ;
 }
-//#endif
+#endif
+
 /** This function formats and prints an ASCII representation of a
     DDS on stdout.  When called by a CGI program, this has the
     effect of sending a DDS object back to the client
@@ -881,7 +851,7 @@ DODSFilter::send_dds(DDS &dds, ConstraintEvaluator &eval,
     send_dds(cout, dds, eval, constrained, anc_location, with_mime_headers);
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 // 'lmt' unused. Should it be used to supply a LMT or removed from the
 // method? jhrg 8/9/05
 void
@@ -895,7 +865,6 @@ DODSFilter::functional_constraint(BaseType &var, DDS &dds,
 
     fflush(out);
 
-    // Grab a stream encodes using XDR.
     XDRFileMarshaller m( out ) ;
 
     try {
@@ -906,7 +875,8 @@ DODSFilter::functional_constraint(BaseType &var, DDS &dds,
         throw;
     }
 }
-//#endif
+#endif
+
 // 'lmt' unused. Should it be used to supply a LMT or removed from the
 // method? jhrg 8/9/05
 void
@@ -931,7 +901,8 @@ DODSFilter::functional_constraint(BaseType &var, DDS &dds,
         throw;
     }
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 void
 DODSFilter::dataset_constraint(DDS & dds, ConstraintEvaluator & eval,
                                FILE * out) const
@@ -956,7 +927,7 @@ DODSFilter::dataset_constraint(DDS & dds, ConstraintEvaluator & eval,
         throw;
     }
 }
-//#endif
+#endif
 
 void
 DODSFilter::dataset_constraint(DDS & dds, ConstraintEvaluator & eval,
@@ -982,7 +953,49 @@ DODSFilter::dataset_constraint(DDS & dds, ConstraintEvaluator & eval,
         throw;
     }
 }
-//#if FILE_METHODS
+
+void
+DODSFilter::dataset_constraint_ddx(DDS & dds, ConstraintEvaluator & eval,
+                               ostream &out, const string &boundary,
+                               const string &start) const
+{
+    // Write the MPM headers for the DDX (text/xml) part of the response
+    set_mime_ddx_boundary(out, boundary, start, dap4_ddx);
+
+    // Make cid
+    uuid_t uu;
+    uuid_generate(uu);
+    char uuid[37];
+    uuid_unparse(uu, &uuid[0]);
+    char domain[256];
+    if (getdomainname(domain, 255) != 0 || strlen(domain) == 0)
+	strncpy(domain, "opendap.org", 255);
+
+    string cid = string(&uuid[0]) + "@" + string(&domain[0]);
+
+    // Send constrained DDX with a data blob reference
+    dds.print_xml(out, true, cid);
+
+    // Write the MPM headers for the data part of the response.
+    set_mime_data_boundary(out, boundary, cid, dap4_data, binary);
+
+    // Grab a stream that encodes using XDR.
+    XDRStreamMarshaller m( out ) ;
+
+    try {
+        // Send all variables in the current projection (send_p())
+        for (DDS::Vars_iter i = dds.var_begin(); i != dds.var_end(); i++)
+            if ((*i)->send_p()) {
+                DBG(cerr << "Sending " << (*i)->name() << endl);
+                (*i)->serialize(eval, dds, m, true);
+            }
+    }
+    catch (Error & e) {
+        throw;
+    }
+}
+
+#if FILE_METHODS
 /** Send the data in the DDS object back to the client program. The data is
     encoded using a Marshaller, and enclosed in a MIME document which is all sent
     to \c data_stream. If this is being called from a CGI, \c data_stream is
@@ -1076,7 +1089,8 @@ DODSFilter::send_data(DDS & dds, ConstraintEvaluator & eval,
 
     fflush(data_stream);
 }
-//#endif
+#endif
+
 /** Send the data in the DDS object back to the client program. The data is
     encoded using a Marshaller, and enclosed in a MIME document which is all sent
     to \c data_stream. If this is being called from a CGI, \c data_stream is
@@ -1118,9 +1132,6 @@ DODSFilter::send_data(DDS & dds, ConstraintEvaluator & eval,
     dds.tag_nested_sequences(); // Tag Sequences as Parent or Leaf node.
 
     // Start sending the response...
-#if COMPRESSION_FOR_SERVER3
-    bool compress = d_comp && deflate_exists();
-#endif
 
     // Handle *functional* constraint expressions specially
     if (eval.functional_expression()) {
@@ -1132,17 +1143,7 @@ DODSFilter::send_data(DDS & dds, ConstraintEvaluator & eval,
         if (!var)
             throw Error(unknown_error, "Error calling the CE function.");
 
-#if COMPRESSION_FOR_SERVER3
-        if (with_mime_headers)
-            set_mime_binary(data_stream, dods_data, d_cgi_ver,
-                            (compress) ? deflate : x_plain, data_lmt);
-	data_stream << flush ;
-
-        int childpid;
-        if (compress)
-            data_stream = compressor(data_stream, childpid);
-#endif
-        if (with_mime_headers)
+       if (with_mime_headers)
             set_mime_binary(data_stream, dods_data, d_cgi_ver, x_plain, data_lmt);
 
 	data_stream << flush ;
@@ -1152,16 +1153,6 @@ DODSFilter::send_data(DDS & dds, ConstraintEvaluator & eval,
         var = 0;
     }
     else {
-#if COMPRESSION_FOR_SERVER3
-        if (with_mime_headers)
-            set_mime_binary(data_stream, dods_data, d_cgi_ver,
-                            (compress) ? deflate : x_plain, data_lmt);
-	data_stream << flush ;
-
-        int childpid;
-        if (compress)
-            data_stream = compressor(data_stream, childpid);
-#endif
         if (with_mime_headers)
             set_mime_binary(data_stream, dods_data, d_cgi_ver, x_plain, data_lmt);
 
@@ -1171,7 +1162,7 @@ DODSFilter::send_data(DDS & dds, ConstraintEvaluator & eval,
     data_stream << flush ;
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 /** Send the DDX response. The DDX never contains data, instead it holds a
     reference to a Blob response which is used to get the data values. The
     DDS and DAS objects are built using code that already exists in the
@@ -1206,10 +1197,11 @@ DODSFilter::send_ddx(DDS &dds, ConstraintEvaluator &eval, FILE *out,
     else {
         if (with_mime_headers)
             set_mime_text(out, dap4_ddx, d_cgi_ver, x_plain, dds_lmt);
-        dds.print_xml(out, !d_ce.empty(), d_url + ".blob?" + d_ce);
+        dds.print_xml(out, !d_ce.empty(), "");
     }
 }
-//#endif
+#endif
+
 /** Send the DDX response. The DDX never contains data, instead it holds a
     reference to a Blob response which is used to get the data values. The
     DDS and DAS objects are built using code that already exists in the
@@ -1244,72 +1236,86 @@ DODSFilter::send_ddx(DDS &dds, ConstraintEvaluator &eval, ostream &out,
     else {
         if (with_mime_headers)
             set_mime_text(out, dap4_ddx, d_cgi_ver, x_plain, dds_lmt);
-        dds.print_xml(out, !d_ce.empty(), d_url + ".blob?" + d_ce);
+        dds.print_xml(out, !d_ce.empty(), "");
     }
 }
-//#if FILE_METHODS
-/** Write the BLOB response to the client.
-    @param dds Use the variables in this DDS to generate the BLOB response.
-    @param out Dump the response to this FILE pointer.
-    @param with_mime_headers If true (the default) send MIME headers. */
-void
-DODSFilter::send_blob(DDS &, FILE *, bool)
-{
-#if 0
-    // Broken. jhrg 4/3/06
-    bool compress = d_comp && deflate_exists();
-    time_t data_lmt = get_data_last_modified_time(d_anc_dir);
 
+/** Send the data in the DDS object back to the client program. The data is
+    encoded using a Marshaller, and enclosed in a MIME document which is all sent
+    to \c data_stream. If this is being called from a CGI, \c data_stream is
+    probably \c stdout and writing to it has the effect of sending the
+    response back to the client.
+
+    @brief Transmit data.
+    @param dds A DDS object containing the data to be sent.
+    @param eval A reference to the ConstraintEvaluator to use.
+    @param data_stream Write the response to this stream.
+    @param anc_location A directory to search for ancillary files (in
+    addition to the CWD).  This is used in a call to
+    get_data_last_modified_time().
+    @param with_mime_headers If true, include the MIME headers in the response.
+    Defaults to true.
+    @return void */
+void
+DODSFilter::send_data_ddx(DDS & dds, ConstraintEvaluator & eval,
+                      ostream & data_stream, const string &start,
+                      const string &boundary, const string & anc_location,
+                      bool with_mime_headers) const
+{
     // If this is a conditional request and the server should send a 304
     // response, do that and exit. Otherwise, continue on and send the full
     // response.
-    if (is_conditional() && data_lmt <= get_request_if_modified_since()
+    time_t data_lmt = get_data_last_modified_time(anc_location);
+    if (is_conditional()
+        && data_lmt <= get_request_if_modified_since()
         && with_mime_headers) {
-        set_mime_not_modified(out);
+        set_mime_not_modified(data_stream);
         return;
     }
+    // Set up the alarm.
+    establish_timeout(data_stream);
+    dds.set_timeout(d_timeout);
 
-    dds.parse_constraint(d_ce);
+    eval.parse_constraint(d_ce, dds);   // Throws Error if the ce doesn't
+					// parse.
+
+    dds.tag_nested_sequences(); // Tag Sequences as Parent or Leaf node.
+
+    // Start sending the response...
 
     // Handle *functional* constraint expressions specially
-    if (dds.functional_expression()) {
-        BaseType *var = dds.eval_function(d_dataset);
+    if (eval.functional_expression()) {
+        BaseType *var = eval.eval_function(dds, d_dataset);
         if (!var)
-            throw Error("Error calling the CE function.");
+            throw Error(unknown_error, "Error calling the CE function.");
 
         if (with_mime_headers)
-            set_mime_binary(out, dods_data, d_cgi_ver,
-                            (compress) ? deflate : x_plain, data_lmt);
+            set_mime_multipart(data_stream, boundary, start, dap4_data_ddx,
+		d_cgi_ver, x_plain, data_lmt);
+	data_stream << flush ;
+	BaseTypeFactory btf;
+	DDS var_dds(&btf, var->name());
+	var->set_send_p(true);
+	var_dds.add_var(var);
+        dataset_constraint_ddx(var_dds, eval, data_stream, boundary, start);
 
-        FILE *comp_sink;
-        XDR *xdr_sink;
-        int childpid = get_sinks(out, compress, &comp_sink, &xdr_sink);
-
-        // In the following call to serialize, suppress CE evaluation.
-        if (!var->serialize(d_dataset, dds, xdr_sink, false))
-            throw Error("Could not send the function result.");
-
-        clean_sinks(childpid, compress, xdr_sink, comp_sink);
+        // functional_constraint_ddx(*var, dds, eval, data_stream, boundary);
+        delete var;
+        var = 0;
     }
     else {
         if (with_mime_headers)
-            set_mime_binary(out, dods_data, d_cgi_ver,
-                            (compress) ? deflate : x_plain, data_lmt);
-
-        FILE *comp_sink;
-        XDR *xdr_sink;
-        int childpid = get_sinks(out, compress, &comp_sink, &xdr_sink);
-
-        for (DDS::Vars_iter i = dds.var_begin(); i != dds.var_end(); i++)
-            if ((*i)->send_p()) // only process projected variables
-                if (!(*i)->serialize(d_dataset, dds, xdr_sink, true))
-                    throw Error(string("Could not serialize variable '")
-                                + (*i)->name() + string("'."));
-
-        clean_sinks(childpid, compress, xdr_sink, comp_sink);
+            set_mime_multipart(data_stream, boundary, start, dap4_data_ddx,
+        	    d_cgi_ver, x_plain, data_lmt);
+        data_stream << flush ;
+        dataset_constraint_ddx(dds, eval, data_stream, boundary, start);
     }
-#endif
+
+    data_stream << flush ;
+
+    if (with_mime_headers)
+	data_stream << CRLF << "--" << boundary << "--" << CRLF;
 }
-//#endif
+
 } // namespace libdap
 
