@@ -46,6 +46,8 @@
 #include "ConstraintEvaluator.h"
 #endif
 
+#define FILE_METHODS 1
+
 namespace libdap
 {
 
@@ -85,6 +87,7 @@ public:
         DDS_Response,
         DataDDS_Response,
         DDX_Response,
+        DataDDX_Response,
         BLOB_Response,
         Version_Response
     };
@@ -171,10 +174,6 @@ public:
 
     virtual void establish_timeout(ostream &stream) const;
 
-    virtual void read_ancillary_das(DAS &das, const string &anc_location = "") const;
-
-    virtual void read_ancillary_dds(DDS &dds, const string &anc_location = "") const;
-
     virtual void print_usage() const;
 
     virtual void send_version_info() const;
@@ -196,14 +195,23 @@ public:
                                        ConstraintEvaluator &eval, ostream &out) const;
     virtual void dataset_constraint(DDS &dds, ConstraintEvaluator &eval,
                                     ostream &out) const;
+    virtual void dataset_constraint_ddx(DDS & dds, ConstraintEvaluator & eval,
+                                   ostream &out, const string &boundary,
+                                   const string &start) const;
+
     virtual void send_data(DDS &dds, ConstraintEvaluator &eval,
                            ostream &data_stream,
                            const string &anc_location = "",
                            bool with_mime_headers = true) const;
     virtual void send_ddx(DDS &dds, ConstraintEvaluator &eval, ostream &out,
                           bool with_mime_headers = true) const;
+    virtual void send_data_ddx(DDS &dds, ConstraintEvaluator &eval,
+                           ostream &data_stream, const string &start,
+                           const string &boundary,
+                           const string &anc_location = "",
+                           bool with_mime_headers = true) const;
 
-    //#if FILE_METHODS
+#if FILE_METHODS
     virtual void establish_timeout(FILE *stream) const;
     virtual void send_das(FILE *out, DAS &das, const string &anc_location = "",
                           bool with_mime_headers = true) const;
@@ -221,9 +229,7 @@ public:
                            bool with_mime_headers = true) const;
     virtual void send_ddx(DDS &dds, ConstraintEvaluator &eval, FILE *out,
                           bool with_mime_headers = true) const;
-    // Broken. 4/5/06 jhrg
-    virtual void send_blob(DDS &dds, FILE *out, bool with_mime_headers = true);
-    //#endif
+#endif
 };
 
 } // namespace libdap
