@@ -35,6 +35,8 @@
 
 #include "config.h"
 
+//#define DODS_DEBUG
+
 #include <functional>
 #include <algorithm>
 
@@ -44,8 +46,8 @@
 #include "util.h"
 #include "InternalErr.h"
 #include "escaping.h"
-//#include "BTIterAdapter.h"
 
+#include "debug.h"
 
 using namespace std;
 
@@ -54,8 +56,12 @@ namespace libdap {
 void
 Grid::_duplicate(const Grid &s)
 {
+    // Clear out any spurious vars in Constructor::_vars
+    _vars.clear(); // [mjohnson 10 Sep 2009]
+
     _array_var = s._array_var->ptr_duplicate();
     _array_var->set_parent(this);
+    _vars.push_back(_array_var); // so the Constructor::Vars_Iter sees it [mjohnson 10 Sep 2009]
 
     Grid &cs = const_cast<Grid &>(s);
 
@@ -63,7 +69,9 @@ Grid::_duplicate(const Grid &s)
         BaseType *btp = (*i)->ptr_duplicate();
         btp->set_parent(this);
         _map_vars.push_back(btp);
+        _vars.push_back(btp); // push all map vectors as weak refs into super::_vars which won't delete them [mjohnson 10 Sep 2009]
     }
+
 }
 
 /** The Grid constructor requires only the name of the variable
@@ -128,8 +136,10 @@ Grid::operator=(const Grid &rhs)
         delete btp ;
     }
 
+    // this doesn't copy Constructor::_vars so...
     dynamic_cast<Constructor &>(*this) = rhs;
 
+    // we do it in here...
     _duplicate(rhs);
 
     return *this;
@@ -335,37 +345,171 @@ Grid::var(const string &n, bool, btp_stack *s)
 void
 Grid::add_var(BaseType *bt, Part part)
 {
-    if (!bt)
+    if (!bt) {
         throw InternalErr(__FILE__, __LINE__,
                           "Passing NULL pointer as variable to be added.");
+    }
+
+    if (part == array && _array_var) {
+      // Avoid leaking memory...  Function is add, not set, so it is an error to call again for the array part.
+      throw InternalErr(__FILE__, __LINE__, "Error: Grid::add_var called with part==Array, but the array was already set!");
+    }
+
+    // Set to the clone of bt if we get that far.
+    BaseType* bt_clone = 0;
 
-    // Jose Garcia
-    // Now we get a copy of the maps or of the array
-    // so the owner of bt which is external to libdap++
-    // is free to deallocate its object.
     switch (part) {
-    case array:
-        _array_var = bt->ptr_duplicate();
-        _array_var->set_parent(this);
-        return;
-    case maps: {
-            BaseType *btp = bt->ptr_duplicate();
-            btp->set_parent(this);
-            _map_vars.push_back(btp);
-            return;
+
+    case array: {
+        // Refactored to use new set_array ([mjohnson 11 nov 2009])
+        Array* p_arr = dynamic_cast<Array*>(bt);
+        // avoid obvious broken semantics
+        if (!p_arr) {
+          throw InternalErr(__FILE__, __LINE__,
+              "Grid::add_var(): with Part==array: object is not an Array!");
         }
-    default:
+        // Add it as a copy to preserve old semantics.  This sets parent too.
+        bt_clone = p_arr->ptr_duplicate();
+        set_array(static_cast<Array*>(bt_clone));
+    }
+    break;
+
+    case maps: {
+            bt_clone = bt->ptr_duplicate();
+            bt_clone->set_parent(this);
+            _map_vars.push_back(bt_clone);
+        }
+    break;
+
+    default: {
         if (!_array_var) {
-            _array_var = bt->ptr_duplicate();
-            _array_var->set_parent(this);
+            // Refactored to use new set_array ([mjohnson 11 nov 2009])
+            Array* p_arr = dynamic_cast<Array*>(bt);
+            // avoid obvious broken semantics
+            if (!p_arr) {
+              throw InternalErr(__FILE__, __LINE__,
+                  "Grid::add_var(): with Part==array: object is not an Array!");
+            }
+            // Add it as a copy to preserve old semantics.  This sets parent too.
+            bt_clone = p_arr->ptr_duplicate();
+            set_array(static_cast<Array*>(bt_clone));
         }
         else {
-            BaseType *btp = bt->ptr_duplicate();
-            btp->set_parent(this);
-            _map_vars.push_back(btp);
+            bt_clone = bt->ptr_duplicate();
+            bt_clone->set_parent(this);
+            _map_vars.push_back(bt_clone);
         }
-        return;
     }
+    break;
+  }// switch
+
+  // if we get ehre without exception, add the cloned object to the superclass variable iterator
+  // mjohnson 10 Sep 2009
+  // Add it to the superclass _vars list so we can iterate on superclass vars
+  if (bt_clone) {
+    _vars.push_back(bt_clone);
+  }
+}
+
+/**
+ * Set the Array part of the Grid to point to the memory
+ * p_new_arr.  Grid takes control of the memory (no copy
+ * is made).
+ * If there already exists an array portion, the old
+ * one will be deleted to avoid leaks.
+ * @param p_new_arr  the object to store as the array
+ *                   part of the grid.
+ */
+void
+Grid::set_array(Array* p_new_arr)
+{
+  if (!p_new_arr) {
+    throw InternalErr(__FILE__, __LINE__,
+        "Grid::set_array(): Cannot set to null!");
+  }
+  // Make sure not same memory, this would be evil.
+  if (p_new_arr == _array_var) {
+      return;
+   }
+  // clean out any old array
+  delete _array_var; _array_var = 0;
+  // Set the new, with parent
+  _array_var = p_new_arr;
+  _array_var->set_parent(this);
+}
+
+/**
+ * Add the given array p_new_map as a new map
+ * vector for the Grid.
+ *
+ * If add_as_copy, p_new_map will be cloned
+ * and the copy added, leaving p_new_map
+ * in the control of the caller.
+ *
+ * If !add_as_copy, p_new_map will be explicitly
+ * added as the new map vector.
+ *
+ * The actual Array* in the Grid will be returned,
+ * either the address of the COPY if add_as_copy,
+ * else p_new_map itself if !add_as_copy.
+ *
+ * It is an exception for p_new_map to be null.
+ *
+ * @param p_new_map  the map we want to add
+ * @param add_as_copy whether to add p_new_map
+ *             explicitly and take onwership of memory
+ *             or to add a clone of it and leave control
+ *             to caller.
+ * @return the actual object stored in the Grid, whether
+ *         p_new_map, or the address of the copy.
+ *
+ */
+Array*
+Grid::add_map(Array* p_new_map, bool add_as_copy)
+{
+  if (!p_new_map) {
+    throw InternalErr(__FILE__, __LINE__,
+        "Grid::add_map(): cannot have p_new_map null!");
+  }
+
+  if (add_as_copy) {
+    p_new_map = static_cast<Array*>(p_new_map->ptr_duplicate());
+  }
+
+  p_new_map->set_parent(this);
+  _map_vars.push_back(p_new_map);
+  _vars.push_back(p_new_map); // allow superclass iter to work as well.
+
+  // return the one that got put into the Grid.
+  return p_new_map;
+}
+
+/**
+ * Add pMap (or a clone if addAsCopy) to the
+ * FRONT of the maps list.  This is needed if
+ * we are preserving Grid semantics but want to
+ * add a new OUTER dimension, whereas add_map
+ * appends to the end making a new INNER dimension.
+ * @param pMap the map to add or copy and add
+ * @param add_copy if true, copy pMap and add the copy.
+ * @return The actual memory stored in the Grid,
+ *      either pMap (if !add_copy) or the ptr to
+ *      the clone (if add_copy).
+ */
+Array*
+Grid::prepend_map(Array* p_new_map, bool add_copy)
+{
+  if (add_copy)
+    {
+      p_new_map = static_cast<Array*>(p_new_map->ptr_duplicate());
+    }
+
+  p_new_map->set_parent(this);
+  _map_vars.insert(_map_vars.begin(), p_new_map);
+  _vars.insert(_vars.begin(), p_new_map); // allow superclass iter to work as well.
+
+   // return the one that got put into the Grid.
+   return p_new_map;
 }
 
 /** @brief Returns the Grid Array.
@@ -465,6 +609,40 @@ Grid::components(bool constrained)
     return comp;
 }
 
+void Grid::transfer_attributes(AttrTable *at_container)
+{
+    AttrTable *at = at_container->get_attr_table(name());
+
+    if (at) {
+	at->set_is_global_attribute(false);
+
+	array_var()->transfer_attributes(at);
+
+	Map_iter map = map_begin();
+	while (map != map_end()) {
+	    (*map)->transfer_attributes(at);
+	    map++;
+	}
+
+	// Trick: If an attribute that's within the container 'at' still has its
+	// is_global_attribute property set, then it's not really a global attr
+	// but instead an attribute that belongs to this Grid.
+	AttrTable::Attr_iter at_p = at->attr_begin();
+	while (at_p != at->attr_end()) {
+	    if (at->is_global_attribute(at_p)) {
+		if (at->get_attr_type(at_p) == Attr_container)
+		    get_attr_table().append_container(new AttrTable(
+			    *at->get_attr_table(at_p)), at->get_name(at_p));
+		else
+		    get_attr_table().append_attr(at->get_name(at_p),
+			    at->get_type(at_p), at->get_attr_vector(at_p));
+	    }
+
+	    at_p++;
+	}
+    }
+}
+
 // When projected (using whatever the current constraint provides in the way
 // of a projection), is the object still a Grid?
 
@@ -531,7 +709,8 @@ Grid::clear_constraint()
     for (Map_iter m = map_begin(); m != map_end(); ++m)
         dynamic_cast<Array&>(*(*m)).clear_constraint();
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 void
 Grid::print_decl(FILE *out, string space, bool print_semi,
                  bool constraint_info, bool constrained)
@@ -539,6 +718,7 @@ Grid::print_decl(FILE *out, string space, bool print_semi,
     if (constrained && !send_p())
         return;
 
+#if 0
     // If we are printing the declaration of a constrained Grid then check for
     // the case where the projection removes all but one component; the
     // resulting object is a simple array.
@@ -556,7 +736,14 @@ Grid::print_decl(FILE *out, string space, bool print_semi,
     // component Grid, send the M components as elements of a Struture.
     // This will preserve the grouping without violating the rules for a
     // Grid.
-    else if (constrained && !projection_yields_grid()) {
+    // else
+#endif
+    // The problem with the above is that if two Grids are projected and each
+    // contain one variable, say a map, and it happens to have the same name
+    // in each Grid, then without the enclosing Structures, the returned dataset
+    // has two variables with the same name at the same lexical level. So I'm
+    // removing the code above.
+    if (constrained && !projection_yields_grid()) {
         fprintf(out, "%sStructure {\n", space.c_str()) ;
 
         _array_var->print_decl(out, space + "    ", true, constraint_info,
@@ -596,12 +783,15 @@ Grid::print_decl(FILE *out, string space, bool print_semi,
 
     if (print_semi)
         fprintf(out, ";\n") ;
-
+#if 0
     // If sending just one comp, skip sending the terminal semicolon, etc.
 exit:
+#endif
+
     return;
 }
-//#endif
+#endif
+
 void
 Grid::print_decl(ostream &out, string space, bool print_semi,
                  bool constraint_info, bool constrained)
@@ -616,6 +806,7 @@ Grid::print_decl(ostream &out, string space, bool print_semi,
     // I replaced the 'true' with the value of 'print_semi' passed in by the
     // caller. This fixes an issue with the intern_data tests and does not
     // seem to break anything else. jhrg 11/9/07
+#if 0
     int projection = components(true);
     if (constrained && projection == 1) {
         _array_var->print_decl(out, space, print_semi /*true*/, constraint_info,
@@ -630,7 +821,11 @@ Grid::print_decl(ostream &out, string space, bool print_semi,
     // component Grid, send the M components as elements of a Structure.
     // This will preserve the grouping without violating the rules for a
     // Grid.
-    else if (constrained && !projection_yields_grid()) {
+    // else
+#endif
+
+    // See comment for the FILE* version of this method.
+    if (constrained && !projection_yields_grid()) {
 	out << space << "Structure {\n" ;
 
         _array_var->print_decl(out, space + "    ", true, constraint_info,
@@ -670,13 +865,15 @@ Grid::print_decl(ostream &out, string space, bool print_semi,
 
     if (print_semi)
 	out << ";\n" ;
-
+#if 0
     // If sending just one comp, skip sending the terminal semicolon, etc.
 exit:
+#endif
+
     return;
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 class PrintMapField : public unary_function<BaseType *, void>
 {
     FILE *d_out;
@@ -710,6 +907,7 @@ Grid::print_xml(FILE *out, string space, bool constrained)
      // I replaced the 'true' with the value of 'print_semi' passed in by the
      // caller. This fixes an issue with the intern_data tests and does not
      // seem to break anything else. jhrg 11/9/07
+#if 0
      int projection = components(true);
      if (constrained && projection == 1) {
          get_attr_table().print_xml(out, space + "    ", constrained);
@@ -723,7 +921,10 @@ Grid::print_xml(FILE *out, string space, bool constrained)
      // component Grid, send the M components as elements of a Structure.
      // This will preserve the grouping without violating the rules for a
      // Grid.
-     else if (constrained && !projection_yields_grid()) {
+     // else
+#endif
+
+     if (constrained && !projection_yields_grid()) {
          fprintf(out, "%s<Structure", space.c_str());
          if (!name().empty())
              fprintf(out, " name=\"%s\"", id2xml(name()).c_str());
@@ -758,7 +959,7 @@ Grid::print_xml(FILE *out, string space, bool constrained)
          fprintf(out, "%s</Grid>\n", space.c_str());
      }
 }
-//#endif
+#endif
 
 class PrintMapFieldStrm : public unary_function<BaseType *, void>
 {
@@ -793,6 +994,7 @@ Grid::print_xml(ostream &out, string space, bool constrained)
     // I replaced the 'true' with the value of 'print_semi' passed in by the
     // caller. This fixes an issue with the intern_data tests and does not
     // seem to break anything else. jhrg 11/9/07
+#if 0
     int projection = components(true);
     if (constrained && projection == 1) {
         get_attr_table().print_xml(out, space + "    ", constrained);
@@ -806,7 +1008,10 @@ Grid::print_xml(ostream &out, string space, bool constrained)
     // component Grid, send the M components as elements of a Structure.
     // This will preserve the grouping without violating the rules for a
     // Grid.
-    else if (constrained && !projection_yields_grid()) {
+    //else
+#endif
+
+    if (constrained && !projection_yields_grid()) {
         out << space << "<Structure" ;
         if (!name().empty())
             out << " name=\"" << id2xml(name()) << "\"" ;
@@ -842,7 +1047,7 @@ Grid::print_xml(ostream &out, string space, bool constrained)
     }
 }
 
-//#if FILE_METHODS
+#if FILE_METHODS
 void
 Grid::print_val(FILE *out, string space, bool print_decl_p)
 {
@@ -872,7 +1077,8 @@ Grid::print_val(FILE *out, string space, bool print_decl_p)
     if (print_decl_p)
         fprintf(out, ";\n") ;
 }
-//#endif
+#endif
+
 void
 Grid::print_val(ostream &out, string space, bool print_decl_p)
 {
@@ -914,11 +1120,6 @@ Grid::check_semantics(string &msg, bool all)
 {
     if (!BaseType::check_semantics(msg))
         return false;
-#if 0
-    // Actually, the spec doesn't say this. jhrg 2/13/06
-    if (!unique_names(_map_vars, name(), type_name(), msg))
-        return false;
-#endif
 
     msg = "";
 
