@@ -37,7 +37,20 @@
 
 #include "config.h"
 
+#include "Byte.h"
+#include "Int16.h"
+#include "UInt16.h"
+#include "Int32.h"
+#include "UInt32.h"
+#include "Float32.h"
+#include "Float64.h"
+#include "Str.h"
+#include "Url.h"
+#include "Array.h"
 #include "Structure.h"
+#include "Sequence.h"
+#include "Grid.h"
+
 #include "util.h"
 #include "debug.h"
 #include "InternalErr.h"
@@ -432,7 +445,8 @@ Structure::m_exact_match(const string &name, btp_stack *s)
 
     return 0;
 }
-//#if FILE_METHODS
+
+#if FILE_METHODS
 void
 Structure::print_val(FILE *out, string space, bool print_decl_p)
 {
@@ -452,7 +466,8 @@ Structure::print_val(FILE *out, string space, bool print_decl_p)
     if (print_decl_p)
         fprintf(out, ";\n") ;
 }
-//#endif
+#endif
+
 void
 Structure::print_val(ostream &out, string space, bool print_decl_p)
 {
