@@ -39,7 +39,7 @@
 #include <cstring>
 
 static char rcsid[] not_used =
-    { "$Id: Vector.cc 19807 2008-11-06 22:46:39Z jimg $"
+    { "$Id: Vector.cc 21976 2010-01-12 17:16:22Z jimg $"
     };
 
 //#define DODS_DEBUG
@@ -51,6 +51,7 @@ static char rcsid[] not_used =
 #include "util.h"
 #include "debug.h"
 #include "InternalErr.h"
+#include <sstream>
 
 using std::cerr;
 using std::endl;
@@ -98,8 +99,127 @@ void Vector::_duplicate(const Vector & v)
     _buf = 0;                   // init to null
     if (v._buf)                 // only copy if data present
         val2buf(v._buf);        // store v's value in this's _BUF.
+
+    _capacity = v._capacity;
 }
 
+/**
+ * @return whether the type of this Vector is a cardinal type
+ * (ie stored in _buf)
+ */
+bool
+Vector::is_cardinal_type() const
+{
+  // Not cardinal if no _var at all!
+  if (!_var) {
+    return false;
+  }
+
+  switch (_var->type()) {
+     case dods_byte_c:
+     case dods_int16_c:
+     case dods_uint16_c:
+     case dods_int32_c:
+     case dods_uint32_c:
+     case dods_float32_c:
+     case dods_float64_c: {
+       return true;
+       break;
+     }
+
+     // These must be handled differently.
+     case dods_str_c:
+     case dods_url_c:
+     case dods_array_c:
+     case dods_structure_c:
+     case dods_sequence_c:
+     case dods_grid_c:
+       return false;
+       break;
+
+     default:
+       cerr << "Vector::var: Unrecognized type" << endl;
+       return false;
+  } // switch
+}
+
+/**
+ * Create _buf so that it can store numElts of the
+ * (assumed) cardinal type.  This create storage for
+ * width() * numElts bytes.
+ * If _buf already exists, this DELETES IT and creates a new one.
+ * So don't use this if you want to keep the original _buf data around.
+ * This also sets the valueCapacity().
+ * @param numElts the number of elements of the cardinal type in var()
+                  that we want storage for.
+ * @return the size of the buffer created.
+ * @exception if the Vector's type is not cardinal type.
+ */
+unsigned int
+Vector::create_cardinal_data_buffer_for_type(unsigned int numEltsOfType)
+{
+  // Make sure we HAVE a _var, or we cannot continue.
+  if (!_var) {
+    throw InternalErr(__FILE__, __LINE__,
+        "create_cardinal_data_buffer_for_type: Logic error: _var is null!");
+  }
+
+  // Make sure we only do this for the correct data types.
+  if (!is_cardinal_type()) {
+      throw InternalErr(__FILE__, __LINE__,
+          "create_cardinal_data_buffer_for_type: incorrectly used on Vector whose type was not a cardinal (simple data types).");
+  }
+
+  delete_cardinal_data_buffer();
+
+  // Actually new up the array with enough bytes to hold numEltsOfType of the actual type.
+  unsigned int bytesPerElt = _var->width();
+  unsigned int bytesNeeded = bytesPerElt * numEltsOfType;
+  _buf = new char[bytesNeeded];
+  if (!_buf) {
+    ostringstream oss;
+    oss << "create_cardinal_data_buffer_for_type: new char[] failed to allocate " <<
+           bytesNeeded <<
+           " bytes!  Out of memory or too large a buffer required!";
+    throw InternalErr(__FILE__, __LINE__, oss.str());
+  }
+  _capacity = numEltsOfType;
+  return bytesNeeded;
+}
+
+/** Delete _buf and zero it and _capacity out */
+void
+Vector::delete_cardinal_data_buffer()
+{
+  if (_buf) {
+     delete[] _buf;
+     _buf = 0;
+     _capacity = 0;
+   }
+}
+
+/** Helper to reduce cut and paste in the virtual's.
+ *
+ */
+template <class CardType>
+void
+Vector::set_cardinal_values_internal(const CardType* fromArray, int numElts)
+{
+  if (numElts < 0) {
+     throw InternalErr(__FILE__, __LINE__,
+                 "Logic error: Vector::set_cardinal_values_internal() called with negative numElts!");
+  }
+  if (!fromArray) {
+    throw InternalErr(__FILE__, __LINE__,
+        "Logic error: Vector::set_cardinal_values_internal() called with null fromArray!");
+  }
+  set_length(numElts);
+  create_cardinal_data_buffer_for_type(numElts);
+  memcpy(_buf, fromArray, numElts * sizeof(CardType) );
+  set_read_p(true);
+}
+
+
 /** The Vector constructor requires the name of the variable to be
     created, and a pointer to an object of the type the Vector is to
     hold.  The name may be omitted, which will create a nameless
@@ -117,7 +237,7 @@ void Vector::_duplicate(const Vector & v)
     @see Type
     @brief The Vector constructor.  */
 Vector::Vector(const string & n, BaseType * v, const Type & t)
-        : BaseType(n, t), _length(-1), _var(0), _buf(0), _vec(0)
+        : BaseType(n, t), _length(-1), _var(0), _buf(0), _vec(0), _capacity(0)
 {
     if (v)
         add_var(v);
@@ -147,7 +267,7 @@ Vector::Vector(const string & n, BaseType * v, const Type & t)
     @see Type
     @brief The Vector constructor.  */
 Vector::Vector(const string & n, const string &d, BaseType * v, const Type & t)
-        : BaseType(n, d, t), _length(-1), _var(0), _buf(0), _vec(0)
+        : BaseType(n, d, t), _length(-1), _var(0), _buf(0), _vec(0), _capacity(0)
 {
     if (v)
         add_var(v);
@@ -174,16 +294,8 @@ Vector::~Vector()
     delete _var;
     _var = 0;
 
-    if (_buf) {
-        delete[]_buf;
-        _buf = 0;
-    }
-    else {
-        for (unsigned int i = 0; i < _vec.size(); ++i) {
-            delete _vec[i];
-            _vec[i] = 0;
-        }
-    }
+    // Clears all buffers
+    clear_local_data();
 
     DBG2(cerr << "Exiting ~Vector" << endl);
 }
@@ -200,6 +312,17 @@ Vector & Vector::operator=(const Vector & rhs)
     return *this;
 }
 
+void
+Vector::set_name(const std::string& name)
+{
+  BaseType::set_name(name);
+  // We need to set the template variable name as well since
+  // this is what gets output in the dds!  Otherwise, there's a mismatch.
+  if (_var) {
+      _var->set_name(name);
+  }
+}
+
 int Vector::element_count(bool leaves)
 {
     if (!leaves)
@@ -234,7 +357,9 @@ void Vector::set_send_p(bool state)
     @brief Indicates that the data is ready to send.  */
 void Vector::set_read_p(bool state)
 {
-    _var->set_read_p(state);
+    if (_var) {
+      _var->set_read_p(state);
+    }
     BaseType::set_read_p(state);
 }
 
@@ -382,9 +507,10 @@ BaseType *Vector::var(unsigned int i)
 unsigned int Vector::width()
 {
     // Jose Garcia
-    if (!_var)
+    if (!_var) {
         throw InternalErr(__FILE__, __LINE__,
                           "Cannot get width since *this* object is not holding data.");
+    }
 
     return length() * _var->width();
 }
@@ -422,6 +548,7 @@ void Vector::set_length(int l)
 void Vector::vec_resize(int l)
 {
     _vec.resize((l > 0) ? l : 0, 0);    // Fill with NULLs
+    _capacity = l; // capacity in terms of number of elements.
 }
 
 /** @brief read data into a variable for later use
@@ -596,10 +723,6 @@ bool Vector::serialize(ConstraintEvaluator & eval, DDS & dds,
 
 bool Vector::deserialize(UnMarshaller &um, DDS * dds, bool reuse)
 {
-#if 0
-    // status no longer used. jhrg 8/28/07
-    bool status;
-#endif
     unsigned int num;
     unsigned i = 0;
 
@@ -611,9 +734,9 @@ bool Vector::deserialize(UnMarshaller &um, DDS * dds, bool reuse)
     case dods_uint32_c:
     case dods_float32_c:
     case dods_float64_c:
-        if (_buf && !reuse)
-            delete[]_buf;
-        _buf = 0;
+        if (_buf && !reuse) {
+            delete_cardinal_data_buffer();
+        }
 
 	um.get_int( (int &)num ) ;
 
@@ -627,7 +750,8 @@ bool Vector::deserialize(UnMarshaller &um, DDS * dds, bool reuse)
             throw InternalErr(__FILE__, __LINE__, "The server sent declarations and data with mismatched sizes.");
 
         if (!_buf) {
-            _buf = new char[width()];   // we always do the allocation!
+            // Make _buf be large enough for length() elements of _var->type()
+            create_cardinal_data_buffer_for_type(length());
             DBG(cerr << "Vector::deserialize: allocating "
                 << width() << " bytes for an array of "
                 << length() << " " << _var->type_name() << endl);
@@ -654,6 +778,7 @@ bool Vector::deserialize(UnMarshaller &um, DDS * dds, bool reuse)
                               "The client sent declarations and data with mismatched sizes.");
 
         d_str.resize((num > 0) ? num : 0);      // Fill with NULLs
+        _capacity = num; // capacity is number of strings we can fit.
 
         for (i = 0; i < num; ++i) {
             string str;
@@ -746,12 +871,11 @@ unsigned int Vector::val2buf(void *val, bool reuse)
             // width() returns the size given the constraint
             unsigned int array_wid = width();
             if (_buf && !reuse) {
-                delete[]_buf;
-                _buf = 0;
+               delete_cardinal_data_buffer();
             }
 
             if (!_buf) {        // First time or no reuse (free'd above)
-                _buf = new char[array_wid];
+              create_cardinal_data_buffer_for_type(length());
             }
 
             memcpy(_buf, val, array_wid);
@@ -763,6 +887,7 @@ unsigned int Vector::val2buf(void *val, bool reuse)
             // Assume val points to an array of C++ string objects. Copy
             // them into the vector<string> field of this object.
             d_str.resize(_length);
+            _capacity = _length;
             for (int i = 0; i < _length; ++i)
                 d_str[i] = *(static_cast < string * >(val) + i);
 
@@ -828,8 +953,13 @@ unsigned int Vector::buf2val(void **val)
     case dods_uint32_c:
     case dods_float32_c:
     case dods_float64_c:
-        if (!*val)
+        if (!*val) {
             *val = new char[wid];
+        }
+        // avoid bus error if _buf is null and this is called improperly.
+        if (!_buf) {
+          throw InternalErr(__FILE__, __LINE__, "Vector::buf2val: Logic error: called when _buf was null!");
+        }
 
         (void) memcpy(*val, _buf, wid);
 
@@ -896,6 +1026,245 @@ void Vector::set_vec(unsigned int i, BaseType * val)
     _vec[i] = val->ptr_duplicate();
 }
 
+/**
+ * Remove any read or set data in the private data of this Vector,
+ * setting read_p() to false.
+ * Essentially clears the _buf, d_str, and _vec of any data.
+ * Useful for tightening up memory when the data is no longer needed,
+ * but the object cannot yet be destroyed.
+ * NOTE: this is not virtual, and only affects the data in Vector itself!
+ * On exit: get_value_capacity() == 0 && !read_p()
+ */
+void
+Vector::clear_local_data()
+{
+  if (_buf) {
+    delete[]_buf;
+    _buf = 0;
+  }
+
+  for (unsigned int i = 0; i < _vec.size(); ++i) {
+    delete _vec[i];
+    _vec[i] = 0;
+  }
+
+  // Force memory to be reclaimed.
+  _vec.resize(0);
+  d_str.resize(0);
+
+  _capacity = 0;
+  set_read_p(false);
+}
+
+
+/**
+ * Return the capacity of the Vector in terms of number of
+ * elements of its data type that it CAN currently hold (i.e. not bytes).
+ * For example, this could be
+ * the size of the _buf array in bytes / sizeof(T) for the cardinal
+ * types T, or the capacity of the d_str vector if T is string or url type.
+*/
+unsigned int
+Vector::get_value_capacity() const
+{
+  return _capacity;
+}
+
+
+/**
+ * Allocate enough memory for the Vector to contain
+ * numElements data elements of the Vector's type.
+ * Must be used before set_value_slice_from_row_major_vector
+ * to ensure memory exists.
+ * @param numElements  the number of elements of the Vector's type
+ *                     to preallocate storage for.
+ * @exception if the memory cannot be allocated
+ */
+void
+Vector::reserve_value_capacity(unsigned int numElements)
+{
+  if (!_var) {
+   throw InternalErr(__FILE__, __LINE__,
+       "reserve_value_capacity: Logic error: _var is null!");
+  }
+  switch (_var->type()) {
+     case dods_byte_c:
+     case dods_int16_c:
+     case dods_uint16_c:
+     case dods_int32_c:
+     case dods_uint32_c:
+     case dods_float32_c:
+     case dods_float64_c: {
+         // Make _buf be the right size and set _capacity
+         create_cardinal_data_buffer_for_type(numElements);
+       }
+       break;
+
+     case dods_str_c:
+     case dods_url_c: {
+         // Make sure the d_str has enough room for all the strings.
+         // Technically not needed, but it will speed things up for large arrays.
+         d_str.reserve(numElements);
+         _capacity = numElements;
+       }
+       break;
+
+
+     case dods_array_c:
+     case dods_structure_c:
+     case dods_sequence_c:
+     case dods_grid_c: {
+         // not clear anyone will go this path, but best to be complete.
+         _vec.reserve(numElements);
+         _capacity = numElements;
+       }
+      break;
+
+     default: {
+         throw InternalErr(__FILE__, __LINE__, "reserve_value_capacity: Unknown type!");
+       }
+     break;
+
+  } // switch
+
+}
+
+/**
+ * Make sure there's storage allocated for the current length()
+ * of the Vector.
+ * Same as reserveValueCapacity(length())
+ */
+void
+Vector::reserve_value_capacity()
+{
+  // Use the current length of the vector as the reserve amount.
+  reserve_value_capacity(length());
+}
+
+/**
+ * Copy rowMajorData.length() elements currently in a rowMajorData buffer
+ * into this value buffer starting at element index startElement and
+ * continuing up to startElement+rowMajorData.length()-1
+ *
+ * This is used for aggregating together smaller rowMajor vectors
+ * into a larger one.
+ *
+ * Note: unlike the other set_value calls, this does NOT set read_p()
+ *       since it is assumed to be used as a partial read and the caller
+ *       is expected to set_read_p() when the data is complete.
+ *
+ * ASSUMES: rowMajorData.read_p() so that the data is valid!
+ * ASSUMES: this Vector has enough value_capacity() to contain
+ *          all the elements such that:
+ *          startElement + rowMajorData.length()
+ *          <= this->value_capacity().
+ * ASSUMES: the data type of this->var() and rowMajorData.var()
+ *          MUST be non-NULL and be the same!
+ *
+ * @param rowMajorData the vector from which to copy data,
+ *                     assumed already read in or set.
+ * @param startElement the element index
+ *                     (NOT byte, but rather data type element)
+ *                     to place the first data value.
+ * @return the number of elements added, such that:
+ *         startElement + the return value is the next "free" element.
+ */
+unsigned int
+Vector::set_value_slice_from_row_major_vector(const Vector& rowMajorDataC, unsigned int startElement)
+{
+  static const string funcName = "set_value_slice_from_row_major_vector:";
+
+  // semantically const from the caller's viewpoint, but some calls are not syntactic const.
+  Vector& rowMajorData = const_cast<Vector&>(rowMajorDataC);
+
+  bool typesMatch = rowMajorData.var() && _var && (rowMajorData.var()->type() == _var->type());
+  if (!typesMatch) {
+    throw InternalErr(__FILE__, __LINE__,
+        funcName + "Logic error: types do not match so cannot be copied!");
+  }
+
+  // Make sure the data exists
+  if (!rowMajorData.read_p()) {
+    throw InternalErr(__FILE__, __LINE__,
+        funcName + "Logic error: the Vector to copy data from has !read_p() and should have been read in!");
+  }
+
+  // Check this otherwise the static_cast<unsigned int> below will do the wrong thing.
+  if (rowMajorData.length() < 0) {
+    throw InternalErr(__FILE__, __LINE__,
+            funcName + "Logic error: the Vector to copy data from has length() < 0 and was probably not initialized!");
+  }
+
+  // The read-in capacity had better be at least the length (the amountt we will copy) or we'll memcpy into bad memory
+  // I imagine we could copy just the capacity rather than throw, but I really think this implies a problem to be addressed.
+  if (rowMajorData.get_value_capacity() < static_cast<unsigned int>(rowMajorData.length())) {
+    throw InternalErr(__FILE__, __LINE__,
+        funcName + "Logic error: the Vector to copy from has a data capacity less than its length, can't copy!");
+  }
+
+  // Make sure there's enough room in this Vector to store all the elements requested.  Again,
+  // better to throw than just copy what we can since it implies a logic error that needs to be solved.
+  if (_capacity < (startElement + rowMajorData.length())) {
+    throw InternalErr(__FILE__, __LINE__,
+        funcName + "Logic error: the capacity of this Vector cannot hold all the data in the from Vector!");
+  }
+
+  // OK, at this point we're pretty sure we can copy the data, but we have to do it differently depending on type.
+  switch (_var->type()) {
+      case dods_byte_c:
+      case dods_int16_c:
+      case dods_uint16_c:
+      case dods_int32_c:
+      case dods_uint32_c:
+      case dods_float32_c:
+      case dods_float64_c: {
+        if (!_buf) {
+          throw InternalErr(__FILE__, __LINE__, funcName + "Logic error: this->_buf was unexpectedly null!");
+        }
+        if (!rowMajorData._buf) {
+          throw InternalErr(__FILE__, __LINE__, funcName + "Logic error: rowMajorData._buf was unexpectedly null!");
+        }
+        // memcpy the data into this, taking care to do ptr arithmetic on bytes and not sizeof(element)
+        int varWidth = _var->width();
+        char* pFromBuf = rowMajorData._buf;
+        int numBytesToCopy = rowMajorData.width();
+        char* pIntoBuf = _buf + (startElement * varWidth);
+        memcpy(pIntoBuf,
+               pFromBuf,
+               numBytesToCopy );
+      }
+      break;
+
+      case dods_str_c:
+      case dods_url_c: {
+        // Strings need to be copied directly
+        for (unsigned int i = 0; i < static_cast<unsigned int>(rowMajorData.length()); ++i) {
+          d_str[startElement + i] = rowMajorData.d_str[i];
+        }
+      }
+      break;
+
+      case dods_array_c:
+      case dods_structure_c:
+      case dods_sequence_c:
+      case dods_grid_c: {
+        // Not sure that this function will be used for these type of nested objects, so I will throw here.
+        // TODO impl and test this path if it's ever needed.
+        throw InternalErr(__FILE__, __LINE__,
+            funcName + "Unimplemented method for Vectors of type: dods_array_c, dods_structure_c, dods_sequence_c and dods_grid_c.");
+      }
+      break;
+
+      default: {
+        throw InternalErr(__FILE__, __LINE__, funcName + ": Unknown type!");
+      }
+      break;
+
+  } // switch (_var->type())
+
+  // This is how many elements we copied.
+  return (unsigned int)rowMajorData.length();
+}
 
 //@{
 /** @brief set the value of a byte array */
@@ -903,10 +1272,8 @@ bool
 Vector::set_value(dods_byte *val, int sz)
 {
     if (var()->type() == dods_byte_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_byte[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_byte));
-        set_read_p(true);
-        return true;
+      set_cardinal_values_internal<dods_byte>(val, sz);
+      return true;
     }
     else {
         return false;
@@ -917,18 +1284,7 @@ Vector::set_value(dods_byte *val, int sz)
 bool
 Vector::set_value(vector<dods_byte> &val, int sz)
 {
-    if (var()->type() == dods_byte_c) {
-	dods_byte *tmp_buf = new dods_byte[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a int16 array */
@@ -936,10 +1292,8 @@ bool
 Vector::set_value(dods_int16 *val, int sz)
 {
     if (var()->type() == dods_int16_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_int16[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_int16));
-        set_read_p(true);
-        return true;
+      set_cardinal_values_internal<dods_int16>(val, sz);
+      return true;
     }
     else {
         return false;
@@ -950,18 +1304,7 @@ Vector::set_value(dods_int16 *val, int sz)
 bool
 Vector::set_value(vector<dods_int16> &val, int sz)
 {
-    if (var()->type() == dods_int16_c) {
-	dods_int16 *tmp_buf = new dods_int16[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a int32 array */
@@ -969,10 +1312,8 @@ bool
 Vector::set_value(dods_int32 *val, int sz)
 {
     if (var()->type() == dods_int32_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_int32[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_int32));
-        set_read_p(true);
-        return true;
+       set_cardinal_values_internal<dods_int32>(val, sz);
+       return true;
     }
     else {
         return false;
@@ -983,18 +1324,7 @@ Vector::set_value(dods_int32 *val, int sz)
 bool
 Vector::set_value(vector<dods_int32> &val, int sz)
 {
-    if (var()->type() == dods_int32_c) {
-	dods_int32 *tmp_buf = new dods_int32[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a uint16 array */
@@ -1002,10 +1332,8 @@ bool
 Vector::set_value(dods_uint16 *val, int sz)
 {
     if (var()->type() == dods_uint16_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_uint16[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_uint16));
-        set_read_p(true);
-        return true;
+      set_cardinal_values_internal<dods_uint16>(val, sz);
+      return true;
     }
     else {
         return false;
@@ -1016,18 +1344,7 @@ Vector::set_value(dods_uint16 *val, int sz)
 bool
 Vector::set_value(vector<dods_uint16> &val, int sz)
 {
-    if (var()->type() == dods_uint16_c) {
-	dods_uint16 *tmp_buf = new dods_uint16[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a uint32 array */
@@ -1035,9 +1352,7 @@ bool
 Vector::set_value(dods_uint32 *val, int sz)
 {
     if (var()->type() == dods_uint32_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_uint32[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_uint32));
-        set_read_p(true);
+        set_cardinal_values_internal<dods_uint32>(val, sz);
         return true;
     }
     else {
@@ -1049,18 +1364,7 @@ Vector::set_value(dods_uint32 *val, int sz)
 bool
 Vector::set_value(vector<dods_uint32> &val, int sz)
 {
-    if (var()->type() == dods_uint32_c) {
-	dods_uint32 *tmp_buf = new dods_uint32[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a float32 array */
@@ -1068,9 +1372,7 @@ bool
 Vector::set_value(dods_float32 *val, int sz)
 {
     if (var()->type() == dods_float32_c && val) {
-        _buf = reinterpret_cast<char*>(new dods_float32[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_float32));
-        set_read_p(true);
+        set_cardinal_values_internal<dods_float32>(val, sz);
         return true;
     }
     else {
@@ -1082,35 +1384,19 @@ Vector::set_value(dods_float32 *val, int sz)
 bool
 Vector::set_value(vector<dods_float32> &val, int sz)
 {
-    if (var()->type() == dods_float32_c) {
-	dods_float32 *tmp_buf = new dods_float32[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a float64 array */
 bool
 Vector::set_value(dods_float64 *val, int sz)
 {
-    if (!val)
-        return false;
-
-    switch (var()->type()) {
-    case dods_float64_c:
-        _buf = reinterpret_cast<char*>(new dods_float64[sz]) ;
-        memcpy(_buf, val, sz * sizeof(dods_float64));
-        set_read_p(true);
-        return true;
-    default:
-        return false;
+    if (var()->type() == dods_float64_c && val) {
+      set_cardinal_values_internal<dods_float64>(val, sz);
+      return true;
+    }
+    else {
+      return false;
     }
 }
 
@@ -1118,18 +1404,7 @@ Vector::set_value(dods_float64 *val, int sz)
 bool
 Vector::set_value(vector<dods_float64> &val, int sz)
 {
-    if (var()->type() == dods_float64_c) {
-	dods_float64 *tmp_buf = new dods_float64[sz] ;
-        _buf = reinterpret_cast<char*>(tmp_buf) ;
-        for (register int t = 0; t < sz; t++) {
-            tmp_buf[t] = val[t] ;
-        }
-        set_read_p(true);
-        return true;
-    }
-    else {
-        return false;
-    }
+    return set_value(&val[0], sz);
 }
 
 /** @brief set the value of a string or url array */
@@ -1138,6 +1413,7 @@ Vector::set_value(string *val, int sz)
 {
     if ((var()->type() == dods_str_c || var()->type() == dods_url_c) && val) {
         d_str.resize(sz);
+        _capacity = sz;
         for (register int t = 0; t < sz; t++) {
             d_str[t] = val[t] ;
         }
@@ -1156,6 +1432,7 @@ Vector::set_value(vector<string> &val, int sz)
 {
     if (var()->type() == dods_str_c || var()->type() == dods_url_c) {
         d_str.resize(sz);
+        _capacity = sz;
         for (register int t = 0; t < sz; t++) {
             d_str[t] = val[t] ;
         }
