@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,57 +54,34 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 /* Substitute the variable and function names.  */
-#define yyparse ce_exprparse
-#define yylex   ce_exprlex
-#define yyerror ce_exprerror
-#define yylval  ce_exprlval
-#define yychar  ce_exprchar
-#define yydebug ce_exprdebug
-#define yynerrs ce_exprnerrs
-
-
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     SCAN_STR = 258,
-     SCAN_WORD = 259,
-     SCAN_EQUAL = 260,
-     SCAN_NOT_EQUAL = 261,
-     SCAN_GREATER = 262,
-     SCAN_GREATER_EQL = 263,
-     SCAN_LESS = 264,
-     SCAN_LESS_EQL = 265,
-     SCAN_REGEXP = 266
-   };
-#endif
-/* Tokens.  */
-#define SCAN_STR 258
-#define SCAN_WORD 259
-#define SCAN_EQUAL 260
-#define SCAN_NOT_EQUAL 261
-#define SCAN_GREATER 262
-#define SCAN_GREATER_EQL 263
-#define SCAN_LESS 264
-#define SCAN_LESS_EQL 265
-#define SCAN_REGEXP 266
-
-
+#define yyparse         ce_exprparse
+#define yylex           ce_exprlex
+#define yyerror         ce_exprerror
+#define yylval          ce_exprlval
+#define yychar          ce_exprchar
+#define yydebug         ce_exprdebug
+#define yynerrs         ce_exprnerrs
 
 
 /* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
 #line 41 "ce_expr.y"
 
 
 #include "config.h"
 
-static char rcsid[] not_used = {"$Id: ce_expr.y 20520 2009-03-06 01:01:24Z jimg $"};
+static char rcsid[] not_used = {"$Id: ce_expr.y 21578 2009-10-02 16:26:26Z jimg $"};
 
 #include <cassert>
 #include <cstdlib>
@@ -120,13 +96,21 @@ static char rcsid[] not_used = {"$Id: ce_expr.y 20520 2009-03-06 01:01:24Z jimg
 #include "ConstraintEvaluator.h"
 
 #include "BaseType.h"
+
+#include "Byte.h"
+#include "Int16.h"
+#include "UInt16.h"
+#include "Int32.h"
+#include "UInt32.h"
+#include "Float32.h"
+#include "Float64.h"
+#include "Str.h"
+#include "Url.h"
 #include "Array.h"
 #include "Structure.h"
 #include "Sequence.h"
 #include "Grid.h"
-#include "Str.h"
-#include "Int32.h"
-#include "Float64.h"
+
 #include "Error.h"
 
 #include "util.h"
@@ -182,6 +166,9 @@ proj_func get_proj_function(const ConstraintEvaluator &eval, const char *name);
 
 
 
+/* Line 189 of yacc.c  */
+#line 171 "ce_expr.tab.cc"
+
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -200,10 +187,34 @@ proj_func get_proj_function(const ConstraintEvaluator &eval, const char *name);
 # define YYTOKEN_TABLE 0
 #endif
 
+
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     SCAN_STR = 258,
+     SCAN_WORD = 259,
+     SCAN_EQUAL = 260,
+     SCAN_NOT_EQUAL = 261,
+     SCAN_GREATER = 262,
+     SCAN_GREATER_EQL = 263,
+     SCAN_LESS = 264,
+     SCAN_LESS_EQL = 265,
+     SCAN_REGEXP = 266
+   };
+#endif
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 122 "ce_expr.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 130 "ce_expr.y"
+
     bool boolean;
     int op;
     char id[ID_MAX];
@@ -218,22 +229,23 @@ typedef union YYSTYPE
     
     libdap::rvalue *rval_ptr;
     libdap::rvalue_list *r_val_l_ptr;
-}
-/* Line 187 of yacc.c.  */
-#line 224 "ce_expr.tab.cc"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 237 "ce_expr.tab.cc"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 237 "ce_expr.tab.cc"
+/* Line 264 of yacc.c  */
+#line 249 "ce_expr.tab.cc"
 
 #ifdef short
 # undef short
@@ -308,14 +320,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -396,9 +408,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -432,12 +444,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -531,11 +543,11 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   163,   163,   168,   170,   170,   174,   180,   181,   187,
-     215,   219,   225,   247,   248,   254,   263,   274,   280,   293,
-     294,   306,   313,   322,   327,   332,   363,   379,   384,   392,
-     398,   409,   410,   432,   436,   442,   454,   472,   498,   499,
-     500,   501,   502,   503,   504
+       0,   171,   171,   176,   178,   178,   182,   188,   189,   195,
+     223,   227,   233,   255,   256,   262,   271,   282,   288,   301,
+     302,   314,   321,   330,   335,   340,   371,   387,   392,   400,
+     406,   417,   418,   440,   444,   450,   462,   480,   506,   507,
+     508,   509,   510,   511,   512
 };
 #endif
 
@@ -547,7 +559,7 @@ static const char *const yytname[] =
   "$end", "error", "$undefined", "SCAN_STR", "SCAN_WORD", "SCAN_EQUAL",
   "SCAN_NOT_EQUAL", "SCAN_GREATER", "SCAN_GREATER_EQL", "SCAN_LESS",
   "SCAN_LESS_EQL", "SCAN_REGEXP", "'&'", "','", "'('", "')'", "'{'", "'}'",
-  "'['", "']'", "':'", "$accept", "constraint_expr", "@1", "projection",
+  "'['", "']'", "':'", "$accept", "constraint_expr", "$@1", "projection",
   "proj_clause", "proj_function", "selection", "clause", "bool_function",
   "r_value", "r_value_list", "arg_list", "id_or_const", "array_projection",
   "array_proj_clause", "name", "array_indices", "array_index", "rel_op", 0
@@ -849,17 +861,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -893,11 +908,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1177,10 +1192,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1196,11 +1209,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1208,9 +1220,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1234,14 +1246,39 @@ yyparse ()
 #endif
 #endif
 {
-  
-  int yystate;
+
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1249,51 +1286,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1323,7 +1337,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1331,7 +1344,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1354,9 +1366,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1367,7 +1378,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1377,6 +1387,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1385,16 +1398,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1426,20 +1439,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1479,7 +1488,9 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 163 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 171 "ce_expr.y"
     {
 		     DDS(arg)->mark_all(true);
 		     (yyval.boolean) = true;
@@ -1487,33 +1498,43 @@ yyreduce:
     break;
 
   case 4:
-#line 170 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 178 "ce_expr.y"
     { DDS(arg)->mark_all(true); ;}
     break;
 
   case 5:
-#line 171 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 179 "ce_expr.y"
     { 
 		     (yyval.boolean) = (yyvsp[(3) - (3)].boolean);
 		 ;}
     break;
 
   case 6:
-#line 175 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 183 "ce_expr.y"
     {
 		     (yyval.boolean) = (yyvsp[(1) - (3)].boolean) && (yyvsp[(3) - (3)].boolean);
 		 ;}
     break;
 
   case 8:
-#line 182 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 190 "ce_expr.y"
     {
 		    (yyval.boolean) = (yyvsp[(1) - (3)].boolean) && (yyvsp[(3) - (3)].boolean);
 		;}
     break;
 
   case 9:
-#line 188 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 196 "ce_expr.y"
     { 
 		    BaseType *var = DDS(arg)->var((yyvsp[(1) - (1)].id));
 		    if (var) {
@@ -1528,21 +1549,27 @@ yyreduce:
     break;
 
   case 10:
-#line 216 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 224 "ce_expr.y"
     {
 		    (yyval.boolean) = (yyvsp[(1) - (1)].boolean);
 		;}
     break;
 
   case 11:
-#line 220 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 228 "ce_expr.y"
     {
 		    (yyval.boolean) = (yyvsp[(1) - (1)].boolean);
 		;}
     break;
 
   case 12:
-#line 226 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 234 "ce_expr.y"
     {
 		    proj_func p_f = 0;
 		    btp_func f = 0;
@@ -1565,14 +1592,18 @@ yyreduce:
     break;
 
   case 14:
-#line 249 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 257 "ce_expr.y"
     {
 		    (yyval.boolean) = (yyvsp[(1) - (3)].boolean) && (yyvsp[(3) - (3)].boolean);
 		;}
     break;
 
   case 15:
-#line 255 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 263 "ce_expr.y"
     {
 		    if ((yyvsp[(1) - (5)].rval_ptr)) {
 			EVALUATOR(arg)->append_clause((yyvsp[(2) - (5)].op), (yyvsp[(1) - (5)].rval_ptr), (yyvsp[(4) - (5)].r_val_l_ptr));
@@ -1584,7 +1615,9 @@ yyreduce:
     break;
 
   case 16:
-#line 264 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 272 "ce_expr.y"
     {
 		    if ((yyvsp[(1) - (3)].rval_ptr)) {
 			rvalue_list *rv = new rvalue_list;
@@ -1598,14 +1631,18 @@ yyreduce:
     break;
 
   case 17:
-#line 275 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 283 "ce_expr.y"
     {
 		    (yyval.boolean) = (yyvsp[(1) - (1)].boolean);
 		;}
     break;
 
   case 18:
-#line 281 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 289 "ce_expr.y"
     {
 		   bool_func b_func = get_function((*EVALUATOR(arg)), (yyvsp[(1) - (4)].id));
 		   if (!b_func) {
@@ -1619,7 +1656,9 @@ yyreduce:
     break;
 
   case 20:
-#line 295 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 303 "ce_expr.y"
     {
 		    btp_func func = get_btp_function((*EVALUATOR(arg)), (yyvsp[(1) - (4)].id));
 		    if (func) {
@@ -1632,7 +1671,9 @@ yyreduce:
     break;
 
   case 21:
-#line 307 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 315 "ce_expr.y"
     {
 		    if ((yyvsp[(1) - (1)].rval_ptr))
 			(yyval.r_val_l_ptr) = make_rvalue_list((yyvsp[(1) - (1)].rval_ptr));
@@ -1642,7 +1683,9 @@ yyreduce:
     break;
 
   case 22:
-#line 314 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 322 "ce_expr.y"
     {
 		    if ((yyvsp[(1) - (3)].r_val_l_ptr) && (yyvsp[(3) - (3)].rval_ptr))
 			(yyval.r_val_l_ptr) = append_rvalue_list((yyvsp[(1) - (3)].r_val_l_ptr), (yyvsp[(3) - (3)].rval_ptr));
@@ -1652,21 +1695,27 @@ yyreduce:
     break;
 
   case 23:
-#line 323 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 331 "ce_expr.y"
     {  
 		  (yyval.r_val_l_ptr) = (yyvsp[(1) - (1)].r_val_l_ptr);
 	      ;}
     break;
 
   case 24:
-#line 327 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 335 "ce_expr.y"
     { 
 		  (yyval.r_val_l_ptr) = 0; 
 	      ;}
     break;
 
   case 25:
-#line 333 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 341 "ce_expr.y"
     { 
 		    BaseType *btp = DDS(arg)->var(www2id(string((yyvsp[(1) - (1)].id))));
 		    if (btp) {
@@ -1700,7 +1749,9 @@ yyreduce:
     break;
 
   case 26:
-#line 364 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 372 "ce_expr.y"
     {
                     if ((yyvsp[(1) - (1)].val).type != dods_str_c || (yyvsp[(1) - (1)].val).v.s == 0 || (yyvsp[(1) - (1)].val).v.s->empty())
                         ce_exprerror("Malformed string", "");
@@ -1717,14 +1768,18 @@ yyreduce:
     break;
 
   case 27:
-#line 380 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 388 "ce_expr.y"
     {
                     (yyval.boolean) = (*DDS(arg)).mark((yyvsp[(1) - (1)].id), true);
                 ;}
     break;
 
   case 28:
-#line 385 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 393 "ce_expr.y"
     {
                     if (!bracket_projection((*DDS(arg)), (yyvsp[(1) - (2)].id), (yyvsp[(2) - (2)].int_ll_ptr)))
                       no_such_ident((yyvsp[(1) - (2)].id), "array, grid or sequence");
@@ -1735,7 +1790,9 @@ yyreduce:
     break;
 
   case 29:
-#line 393 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 401 "ce_expr.y"
     {
                     string name = string((yyvsp[(1) - (2)].id)) + string ((yyvsp[(2) - (2)].id));
                     strncpy((yyval.id), name.c_str(), ID_MAX-1);
@@ -1744,7 +1801,9 @@ yyreduce:
     break;
 
   case 30:
-#line 399 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 407 "ce_expr.y"
     {
                     string name = string((yyvsp[(1) - (3)].id)) + string ((yyvsp[(2) - (3)].id));
                     if (!bracket_projection((*DDS(arg)), name.c_str(), (yyvsp[(3) - (3)].int_ll_ptr)))
@@ -1756,7 +1815,9 @@ yyreduce:
     break;
 
   case 32:
-#line 411 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 419 "ce_expr.y"
     {
                     if ((yyvsp[(1) - (1)].val).type != dods_str_c || (yyvsp[(1) - (1)].val).v.s == 0 || (yyvsp[(1) - (1)].val).v.s->empty())
                         ce_exprerror("Malformed string", "");
@@ -1779,21 +1840,27 @@ yyreduce:
     break;
 
   case 33:
-#line 433 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 441 "ce_expr.y"
     {
 		    (yyval.int_ll_ptr) = make_array_indices((yyvsp[(1) - (1)].int_l_ptr));
 		;}
     break;
 
   case 34:
-#line 437 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 445 "ce_expr.y"
     {
 		    (yyval.int_ll_ptr) = append_array_index((yyvsp[(1) - (2)].int_ll_ptr), (yyvsp[(2) - (2)].int_l_ptr));
 		;}
     break;
 
   case 35:
-#line 443 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 451 "ce_expr.y"
     {
 		    if (!check_uint32((yyvsp[(2) - (3)].id))) {
 			string msg = "The word `";
@@ -1808,7 +1875,9 @@ yyreduce:
     break;
 
   case 36:
-#line 455 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 463 "ce_expr.y"
     {
 		    if (!check_uint32((yyvsp[(2) - (5)].id))) {
 			string msg = "The word `";
@@ -1829,7 +1898,9 @@ yyreduce:
     break;
 
   case 37:
-#line 473 "ce_expr.y"
+
+/* Line 1455 of yacc.c  */
+#line 481 "ce_expr.y"
     {
 		    if (!check_uint32((yyvsp[(2) - (7)].id))) {
 			string msg = "The word `";
@@ -1856,8 +1927,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1861 "ce_expr.tab.cc"
+
+/* Line 1455 of yacc.c  */
+#line 1933 "ce_expr.tab.cc"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1868,7 +1940,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -1933,7 +2004,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -1950,7 +2021,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2007,9 +2078,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -2034,7 +2102,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2045,7 +2113,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2071,7 +2139,9 @@ yyreturn:
 }
 
 
-#line 507 "ce_expr.y"
+
+/* Line 1675 of yacc.c  */
+#line 515 "ce_expr.y"
 
 
 // All these error reporting function now throw instances of Error. The expr
@@ -2162,8 +2232,10 @@ bool
 bracket_projection(DDS &table, const char *name, int_list_list *indices)
 {
     BaseType *var = table.var(name);
-    Sequence *seq;              // used in last else-if clause
-    
+    Sequence *seq;		// used in last else-if clause
+#if 0
+    Array *array;
+#endif    
     if (!var)
 	return false;
 	
