@@ -36,7 +36,7 @@
 #include "config.h"
 
 static char rcsid[]not_used =
-{   "$Id: ce_functions.cc 20518 2009-03-05 23:39:46Z jimg $"
+{   "$Id: ce_functions.cc 22560 2010-04-21 21:22:00Z jimg $"
 };
 
 #include <limits.h>
@@ -48,10 +48,20 @@ static char rcsid[]not_used =
 #include <vector>
 #include <algorithm>
 
-// #define DODS_DEBUG
+//#define DODS_DEBUG
 
 #include "BaseType.h"
+#include "Byte.h"
+#include "Int16.h"
+#include "UInt16.h"
+#include "Int32.h"
+#include "UInt32.h"
+#include "Float32.h"
+#include "Float64.h"
+#include "Str.h"
+#include "Url.h"
 #include "Array.h"
+#include "Structure.h"
 #include "Sequence.h"
 #include "Grid.h"
 #include "Error.h"
@@ -156,7 +166,9 @@ template<class T> static void set_array_using_double_helper(Array * a,
 void set_array_using_double(Array * dest, double *src, int src_len)
 {
     // Simple types are Byte, ..., Float64, String and Url.
-    if (dest->type() == dods_array_c && !dest->var()->is_simple_type() || dest->var()->type() == dods_str_c || dest->var()->type() == dods_url_c)
+    if ((dest->type() == dods_array_c && !dest->var()->is_simple_type()) 
+	|| dest->var()->type() == dods_str_c 
+	|| dest->var()->type() == dods_url_c)
         throw InternalErr(__FILE__, __LINE__,
                 "The function requires a DAP numeric-type array argument.");
 
@@ -227,7 +239,8 @@ template<class T> static double *extract_double_array_helper(Array * a)
 double *extract_double_array(Array * a)
 {
     // Simple types are Byte, ..., Float64, String and Url.
-    if (a->type() == dods_array_c && !a->var()->is_simple_type() || a->var()->type() == dods_str_c || a->var()->type() == dods_url_c)
+    if ((a->type() == dods_array_c && !a->var()->is_simple_type())
+	|| a->var()->type() == dods_str_c || a->var()->type() == dods_url_c)
         throw Error(malformed_expr,
                 "The function requires a DAP numeric-type array argument.");
 
@@ -314,12 +327,13 @@ function_version(int, BaseType *[], DDS &, BaseType **btpp)
             xml_value =
                     "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
                        <functions>\
-                       <function name=\"version\" version=\"1.0\"/>\
+                       <function name=\"geogrid\" version=\"1.1\"/>\
                        <function name=\"grid\" version=\"1.0\"/>\
-                       <function name=\"geogrid\" version=\"1.0b2\"/>\
-                       <function name=\"geoarray\" version=\"0.9b1\"/>\
                        <function name=\"linear_scale\" version=\"1.0b1\"/>\
-                       </functions>";
+                       <function name=\"version\" version=\"1.0\"/>\
+                     </functions>";
+
+    //                        <function name=\"geoarray\" version=\"0.9b1\"/>
 
     Str *response = new Str("version");
 
@@ -430,19 +444,10 @@ function_grid(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 {
     DBG(cerr << "Entering function_grid..." << endl);
 
-    string
-            info =
-                    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
-        <function name=\"grid\" version=\"1.0\"/>\
-        The grid() function takes a grid variable and zero or more relational\
-        expressions. Each relational expression is applied to the grid using the\
-        server's constraint evaluator and the resulting grid is returned. The\
-        expressions may use constants and the grid's map vectors but may not use\
-        any other variables. Two forms of expression are provide: \"var relop const\"\
-        and \"const relop var relop const\". For example: grid(sst, \"10<=TIME<20\")\
-        and grid(sst, \"10<=TIME\", \"TIME<20\") are both legal and, in this case,\
-        also equivalent.\
-        </function>";
+    string info =
+    string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n") +
+    "<function name=\"grid\" version=\"1.0\" href=\"http://docs.opendap.org/index.php/Server_Side_Processing_Functions#grid\">\n" +
+    "</function>\n";
 
     if (argc == 0) {
         Str *response = new Str("info");
@@ -539,21 +544,10 @@ function_grid(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 void
 function_geogrid(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 {
-    string
-            info =
-                    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
-        <function name=\"geogrid\" version=\"1.0b2\"/>\
-        geogrid() applies a constraint given in latitude and longitude to a\
-        DAP Grid variable. The arguments to the function are:\
-        geogrid(<grid variable>, <upper latitude>, <left longitude>,\
-        <lower latitude>, <right longitude> [selection expressions - see grid()])\
-        geogrid() returns the version of the function.\
-        The function will always return a single Grid variable whose values\
-        completely cover the given region, although there may be cases when\
-        some additional data is also returned. If the longitude values 'wrap\
-        around' the right edge of the data, then the function will make two\
-        requests and return those joined together as a single Grid.\
-        </function>";
+    string info =
+    string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n") +
+    "<function name=\"geogrid\" version=\"1.1\" href=\"http://docs.opendap.org/index.php/Server_Side_Processing_Functions#geogrid\">\n"+
+    "</function>";
 
     if (argc == 0) {
         Str *response = new Str("version");
@@ -582,11 +576,13 @@ function_geogrid(int argc, BaseType * argv[], DDS &, BaseType **btpp)
     Grid::Map_iter i = l_grid->map_begin();
     while (i != l_grid->map_end())
         (*i++)->set_send_p(true);
+
     l_grid->read();
     // Calling read() above sets the read_p flag for the entire grid; clear it
     // for the grid's array so that later on the code will be sure to read it
     // under all circumstances.
-    l_grid->get_array()->set_read_p(false);DBG(cerr << "geogrid: past map read" << endl);
+    l_grid->get_array()->set_read_p(false);
+    DBG(cerr << "geogrid: past map read" << endl);
 
     // Look for Grid Selection Expressions tacked onto the end of the BB
     // specification. If there are any, evaluate them before evaluating the BB.
@@ -616,7 +612,7 @@ function_geogrid(int argc, BaseType * argv[], DDS &, BaseType **btpp)
         double left = extract_double_value(argv[2]);
         double bottom = extract_double_value(argv[3]);
         double right = extract_double_value(argv[4]);
-        gc.set_bounding_box(left, top, right, bottom);
+        gc.set_bounding_box(top, left, bottom, right);
         DBG(cerr << "geogrid: past bounding box set" << endl);
 
         // This also reads all of the data into the grid variable
@@ -751,22 +747,10 @@ static double get_missing_value(BaseType *var)
 void
 function_linear_scale(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 {
-    string
-            info =
-                    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
-        <function name=\"linear_scale\" version=\"1.0b1\">\
-        The linear_scale() function applies the familiar y=mx+b equation to data.\
-        If only the name of a variable is given, the function looks for the COARDS\
-        scale_factor, add_offset and missing_value attributes. In the equation,\
-        'm' is scale_factor, 'b' is add_offset and data values which match\
-        missing_value are not scaled. If add_offset cannot be found, it defaults to\
-        zero; if missing_value cannot be found, the test for it is not performed.\
-        The caller can also provide values for these and, if provided, those values\
-        are used and the dataset's attributes are ignored. If only scale_factor is\
-        provided, the defaults for 'b' and the missing value flag are used.\
-        Similarly, if only 'm' and 'b' are provided the missing value test is not\
-        performed. The values are always returned in a Float64 array.\
-        </function>";
+    string info =
+    string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n") +
+    "<function name=\"linear_scale\" version=\"1.0b1\" href=\"http://docs.opendap.org/index.php/Server_Side_Processing_Functions#linear_scale\">\n" +
+    "</function>";
 
     if (argc == 0) {
         Str *response = new Str("info");
@@ -894,7 +878,7 @@ function_linear_scale(int argc, BaseType * argv[], DDS &, BaseType **btpp)
     *btpp = dest;
     return;
 }
-
+#if 0
 /** Perform a selection on the array using geographical coordinates. This
  function takes several groups of arguments.
  <ul>
@@ -914,19 +898,10 @@ function_linear_scale(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 void
 function_geoarray(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 {
-    string
-            info =
-                    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
-        <function name=\"geoarray\" version=\"0.9b1\"/>\
-        The geoarray() function supports two different sets of arguments:\
-        geoarray(var,left,top,right,bottom)\
-        geoarray(var,left,top,right,bottom,var_left,var_top,var_right,var_bottom)\
-        In the first version 'var' is the target of the selection and 'left', 'top',\
-        'right' and 'bottom' are the corners of a longitude-latitude box that defines\
-        the selection. In the second version the 'var_left', ..., parameters give the\
-        longitude and latitude extent of the entire array. The projection and datum are\
-        assumed to be Plat-Carre and WGS84.\
-        </function>";
+    string info =
+    string("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n") +
+    "<function name=\"geoarray\" version=\"0.9b1\" href=\"http://docs.opendap.org/index.php/Server_Side_Processing_Functions#geoarray\">\n" +
+    "</function>";
 
     if (argc == 0) {
         Str *response = new Str("version");
@@ -1012,13 +987,15 @@ function_geoarray(int argc, BaseType * argv[], DDS &, BaseType **btpp)
 
     throw InternalErr(__FILE__, __LINE__, "Impossible condition in geoarray.");
 }
-
+#endif
 void register_functions(ConstraintEvaluator & ce)
 {
     ce.add_function("grid", function_grid);
     ce.add_function("geogrid", function_geogrid);
     ce.add_function("linear_scale", function_linear_scale);
+#if 0
     ce.add_function("geoarray", function_geoarray);
+#endif
     ce.add_function("version", function_version);
 }
 
