@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,17 +54,121 @@
 /* Pure parsers.  */
 #define YYPURE 0
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 /* Substitute the variable and function names.  */
-#define yyparse ddsparse
-#define yylex   ddslex
-#define yyerror ddserror
-#define yylval  ddslval
-#define yychar  ddschar
-#define yydebug ddsdebug
-#define yynerrs ddsnerrs
+#define yyparse         ddsparse
+#define yylex           ddslex
+#define yyerror         ddserror
+#define yylval          ddslval
+#define yychar          ddschar
+#define yydebug         ddsdebug
+#define yynerrs         ddsnerrs
+
+
+/* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
+#line 47 "dds.y"
+
+
+#include "config_dap.h"
+
+static char rcsid[] not_used = {"$Id: dds.y 21577 2009-10-02 16:12:17Z jimg $"};
+
+#include <cstring>
+#include <cassert>
+#include <iostream>
+#include <stack>
+#include <sstream>
+
+#include "Byte.h"
+#include "Int16.h"
+#include "UInt16.h"
+#include "Int32.h"
+#include "UInt32.h"
+#include "Float32.h"
+#include "Float64.h"
+#include "Str.h"
+#include "Url.h"
+#include "Array.h"
+#include "Structure.h"
+#include "Sequence.h"
+#include "Grid.h"
+
+#include "DDS.h"
+#include "Error.h"
+#include "parser.h"
+#include "util.h"
+
+using namespace std;
+using namespace libdap;
+
+// These macros are used to access the `arguments' passed to the parser. A
+// pointer to an error object and a pointer to an integer status variable are
+// passed in to the parser within a structure (which itself is passed as a
+// pointer). Note that the ERROR macro explicitly casts OBJ to an ERROR *. 
+// ERROR is no longer used. These parsers now signal problems by throwing
+// exceptions. 5/22/2002 jhrg
+#define DDS_OBJ(arg) ((DDS *)((parser_arg *)(arg))->_object)
+
+#define YYPARSE_PARAM arg
+
+extern int dds_line_num;	/* defined in dds.lex */
+
+// No global static objects in the dap library! 1/24/2000 jhrg
+static stack<BaseType *> *ctor;
+static BaseType *current;
+static string *id;
+static Part part = nil;		/* Part is defined in BaseType */
+
+static const char *NO_DDS_MSG =
+"The descriptor object returned from the dataset was null.\n\
+Check that the URL is correct.";
+
+static const char *BAD_DECLARATION =
+"In the dataset descriptor object: Expected a variable declaration\n\
+(e.g., Int32 i;). Make sure that the variable name is not the name\n\
+of a datatype and that the Array: and Maps: sections of a Grid are\n\
+labeled properly.";
+ 
+int ddslex();
+void ddserror(char *s);
+void error_exit_cleanup();
+void add_entry(DDS &table, stack<BaseType *> **ctor, BaseType **current, 
+	       Part p);
+void invalid_declaration(parser_arg *arg, string semantic_err_msg, 
+			 char *type, char *name);
+
+
+
+/* Line 189 of yacc.c  */
+#line 154 "dds.tab.cc"
+
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+
+/* Enabling verbose error messages.  */
+#ifdef YYERROR_VERBOSE
+# undef YYERROR_VERBOSE
+# define YYERROR_VERBOSE 1
+#else
+# define YYERROR_VERBOSE 0
+#endif
+
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
 
 
 /* Tokens.  */
@@ -92,128 +195,35 @@
      SCAN_URL = 273
    };
 #endif
-/* Tokens.  */
-#define SCAN_WORD 258
-#define SCAN_DATASET 259
-#define SCAN_LIST 260
-#define SCAN_SEQUENCE 261
-#define SCAN_STRUCTURE 262
-#define SCAN_FUNCTION 263
-#define SCAN_GRID 264
-#define SCAN_BYTE 265
-#define SCAN_INT16 266
-#define SCAN_UINT16 267
-#define SCAN_INT32 268
-#define SCAN_UINT32 269
-#define SCAN_FLOAT32 270
-#define SCAN_FLOAT64 271
-#define SCAN_STRING 272
-#define SCAN_URL 273
 
 
 
-
-/* Copy the first part of user declarations.  */
-#line 47 "dds.y"
-
-
-#include "config_dap.h"
-
-static char rcsid[] not_used = {"$Id: dds.y 20091 2009-01-29 00:02:40Z jimg $"};
-
-#include <cstring>
-#include <cassert>
-#include <iostream>
-#include <stack>
-#include <sstream>
-
-#include "DDS.h"
-#include "Array.h"
-#include "Error.h"
-#include "parser.h"
-#include "util.h"
-
-using namespace std;
-using namespace libdap;
-
-// These macros are used to access the `arguments' passed to the parser. A
-// pointer to an error object and a pointer to an integer status variable are
-// passed in to the parser within a structure (which itself is passed as a
-// pointer). Note that the ERROR macro explicitly casts OBJ to an ERROR *. 
-// ERROR is no longer used. These parsers now signal problems by throwing
-// exceptions. 5/22/2002 jhrg
-#define DDS_OBJ(arg) ((DDS *)((parser_arg *)(arg))->_object)
-
-#define YYPARSE_PARAM arg
-
-extern int dds_line_num;	/* defined in dds.lex */
-
-// No global static objects in the dap library! 1/24/2000 jhrg
-static stack<BaseType *> *ctor;
-static BaseType *current;
-static string *id;
-static Part part = nil;		/* Part is defined in BaseType */
-
-static char *NO_DDS_MSG =
-"The descriptor object returned from the dataset was null.\n\
-Check that the URL is correct.";
-
-static char *BAD_DECLARATION =
-"In the dataset descriptor object: Expected a variable declaration\n\
-(e.g., Int32 i;). Make sure that the variable name is not the name\n\
-of a datatype and that the Array: and Maps: sections of a Grid are\n\
-labeled properly.";
- 
-int ddslex();
-void ddserror(char *s);
-void error_exit_cleanup();
-void add_entry(DDS &table, stack<BaseType *> **ctor, BaseType **current, 
-	       Part p);
-void invalid_declaration(parser_arg *arg, string semantic_err_msg, 
-			 char *type, char *name);
-
-
-
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-
-/* Enabling verbose error messages.  */
-#ifdef YYERROR_VERBOSE
-# undef YYERROR_VERBOSE
-# define YYERROR_VERBOSE 1
-#else
-# define YYERROR_VERBOSE 0
-#endif
-
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
-#endif
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 108 "dds.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 121 "dds.y"
+
     bool boolean;
     char word[ID_MAX];
-}
-/* Line 187 of yacc.c.  */
-#line 204 "dds.tab.cc"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 215 "dds.tab.cc"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 217 "dds.tab.cc"
+/* Line 264 of yacc.c  */
+#line 227 "dds.tab.cc"
 
 #ifdef short
 # undef short
@@ -288,14 +298,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -376,9 +386,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -412,12 +422,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -515,12 +525,12 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   137,   137,   137,   147,   148,   151,   155,   165,   169,
-     170,   177,   200,   199,   220,   219,   240,   252,   264,   239,
-     285,   296,   302,   308,   314,   315,   316,   317,   318,   319,
-     320,   321,   322,   325,   326,   329,   329,   329,   329,   329,
-     330,   330,   330,   330,   331,   331,   331,   331,   332,   335,
-     359,   363,   358,   390,   401,   402,   403
+       0,   150,   150,   150,   160,   161,   164,   168,   178,   182,
+     183,   190,   213,   212,   233,   232,   253,   265,   277,   252,
+     298,   309,   315,   321,   327,   328,   329,   330,   331,   332,
+     333,   334,   335,   338,   339,   342,   342,   342,   342,   342,
+     343,   343,   343,   343,   344,   344,   344,   344,   345,   348,
+     372,   376,   371,   403,   414,   415,   416
 };
 #endif
 
@@ -533,10 +543,10 @@ static const char *const yytname[] =
   "SCAN_SEQUENCE", "SCAN_STRUCTURE", "SCAN_FUNCTION", "SCAN_GRID",
   "SCAN_BYTE", "SCAN_INT16", "SCAN_UINT16", "SCAN_INT32", "SCAN_UINT32",
   "SCAN_FLOAT32", "SCAN_FLOAT64", "SCAN_STRING", "SCAN_URL", "'{'", "'}'",
-  "';'", "':'", "'['", "']'", "'='", "$accept", "start", "@1", "datasets",
-  "dataset", "declarations", "declaration", "@2", "@3", "@4", "@5", "@6",
-  "structure", "sequence", "grid", "base_type", "var", "var_name",
-  "array_decl", "@7", "@8", "name", 0
+  "';'", "':'", "'['", "']'", "'='", "$accept", "start", "$@1", "datasets",
+  "dataset", "declarations", "declaration", "$@2", "$@3", "$@4", "$@5",
+  "$@6", "structure", "sequence", "grid", "base_type", "var", "var_name",
+  "array_decl", "$@7", "$@8", "name", 0
 };
 #endif
 
@@ -871,17 +881,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -915,11 +928,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1199,10 +1212,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1218,11 +1229,10 @@ int yyparse ();
 #endif /* ! YYPARSE_PARAM */
 
 
-
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -1230,9 +1240,9 @@ int yynerrs;
 
 
 
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1256,14 +1266,39 @@ yyparse ()
 #endif
 #endif
 {
-  
-  int yystate;
+
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1271,51 +1306,28 @@ yyparse ()
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1345,7 +1357,6 @@ yyparse ()
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1353,7 +1364,6 @@ yyparse ()
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1376,9 +1386,8 @@ yyparse ()
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1389,7 +1398,6 @@ yyparse ()
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1399,6 +1407,9 @@ yyparse ()
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1407,16 +1418,16 @@ yyparse ()
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1448,20 +1459,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1501,7 +1508,9 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 137 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 150 "dds.y"
     {
 		    /* On entry to the parser, make the BaseType stack. */
 		    ctor = new stack<BaseType *>;
@@ -1509,21 +1518,27 @@ yyreduce:
     break;
 
   case 3:
-#line 142 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 155 "dds.y"
     {
 		    delete ctor; ctor = 0;
 		;}
     break;
 
   case 6:
-#line 152 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 165 "dds.y"
     {
 		    (yyval.boolean) = (yyvsp[(3) - (6)].boolean) && (yyvsp[(5) - (6)].word);
 		;}
     break;
 
   case 7:
-#line 156 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 169 "dds.y"
     {
 		    parse_error((parser_arg *)arg, NO_DDS_MSG,
  				dds_line_num, (yyvsp[(1) - (1)].word));
@@ -1533,24 +1548,32 @@ yyreduce:
     break;
 
   case 8:
-#line 165 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 178 "dds.y"
     {
 		    (yyval.boolean) = true;
 		;}
     break;
 
   case 9:
-#line 169 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 182 "dds.y"
     { (yyval.boolean) = true; ;}
     break;
 
   case 10:
-#line 170 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 183 "dds.y"
     { (yyval.boolean) = true; ;}
     break;
 
   case 11:
-#line 178 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 191 "dds.y"
     { 
 		    string smsg;
 		    if (current->check_semantics(smsg)) {
@@ -1574,7 +1597,9 @@ yyreduce:
     break;
 
   case 12:
-#line 200 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 213 "dds.y"
     { 
 		    if( current ) delete current ;
 		    current = ctor->top(); 
@@ -1583,7 +1608,9 @@ yyreduce:
     break;
 
   case 13:
-#line 206 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 219 "dds.y"
     { 
 		    string smsg;
 		    if (current->check_semantics(smsg))
@@ -1599,7 +1626,9 @@ yyreduce:
     break;
 
   case 14:
-#line 220 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 233 "dds.y"
     { 
 		    if( current ) delete current ;
 		    current = ctor->top(); 
@@ -1608,7 +1637,9 @@ yyreduce:
     break;
 
   case 15:
-#line 226 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 239 "dds.y"
     { 
 		    string smsg;
 		    if (current->check_semantics(smsg))
@@ -1624,7 +1655,9 @@ yyreduce:
     break;
 
   case 16:
-#line 240 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 253 "dds.y"
     { 
 		    if (is_keyword(string((yyvsp[(3) - (4)].word)), "array"))
 			part = array; 
@@ -1639,7 +1672,9 @@ yyreduce:
     break;
 
   case 17:
-#line 252 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 265 "dds.y"
     { 
 		    if (is_keyword(string((yyvsp[(7) - (8)].word)), "maps"))
 			part = maps; 
@@ -1654,7 +1689,9 @@ yyreduce:
     break;
 
   case 18:
-#line 264 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 277 "dds.y"
     {
 		    if( current ) delete current ;
 		    current = ctor->top(); 
@@ -1663,7 +1700,9 @@ yyreduce:
     break;
 
   case 19:
-#line 270 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 283 "dds.y"
     {
 		    string smsg;
 		    if (current->check_semantics(smsg)) {
@@ -1681,7 +1720,9 @@ yyreduce:
     break;
 
   case 20:
-#line 286 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 299 "dds.y"
     {
 		    ostringstream msg;
 		    msg << BAD_DECLARATION;
@@ -1692,78 +1733,106 @@ yyreduce:
     break;
 
   case 21:
-#line 297 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 310 "dds.y"
     { 
 		    ctor->push(DDS_OBJ(arg)->get_factory()->NewStructure()); 
 		;}
     break;
 
   case 22:
-#line 303 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 316 "dds.y"
     { 
 		    ctor->push(DDS_OBJ(arg)->get_factory()->NewSequence()); 
 		;}
     break;
 
   case 23:
-#line 309 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 322 "dds.y"
     { 
 		    ctor->push(DDS_OBJ(arg)->get_factory()->NewGrid()); 
 		;}
     break;
 
   case 24:
-#line 314 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 327 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewByte(); ;}
     break;
 
   case 25:
-#line 315 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 328 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewInt16(); ;}
     break;
 
   case 26:
-#line 316 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 329 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewUInt16(); ;}
     break;
 
   case 27:
-#line 317 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 330 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewInt32(); ;}
     break;
 
   case 28:
-#line 318 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 331 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewUInt32(); ;}
     break;
 
   case 29:
-#line 319 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 332 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewFloat32(); ;}
     break;
 
   case 30:
-#line 320 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 333 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewFloat64(); ;}
     break;
 
   case 31:
-#line 321 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 334 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewStr(); ;}
     break;
 
   case 32:
-#line 322 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 335 "dds.y"
     { if( current ) delete current ;current = DDS_OBJ(arg)->get_factory()->NewUrl(); ;}
     break;
 
   case 33:
-#line 325 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 338 "dds.y"
     { current->set_name((yyvsp[(1) - (1)].word)); ;}
     break;
 
   case 49:
-#line 336 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 349 "dds.y"
     { 
 		     if (!check_int32((yyvsp[(2) - (3)].word))) {
 			 string msg = "In the dataset descriptor object:\n";
@@ -1788,14 +1857,18 @@ yyreduce:
     break;
 
   case 50:
-#line 359 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 372 "dds.y"
     {
 		     id = new string((yyvsp[(2) - (2)].word));
 		 ;}
     break;
 
   case 51:
-#line 363 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 376 "dds.y"
     { 
 		     if (!check_int32((yyvsp[(5) - (5)].word))) {
 			 string msg = "In the dataset descriptor object:\n";
@@ -1821,14 +1894,18 @@ yyreduce:
     break;
 
   case 52:
-#line 386 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 399 "dds.y"
     {
 		     (yyval.boolean) = true;
 		 ;}
     break;
 
   case 53:
-#line 391 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 404 "dds.y"
     {
 		     ostringstream msg;
 		     msg << "In the dataset descriptor object:" << endl
@@ -1840,17 +1917,23 @@ yyreduce:
     break;
 
   case 54:
-#line 401 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 414 "dds.y"
     { (*DDS_OBJ(arg)).set_dataset_name((yyvsp[(1) - (1)].word)); ;}
     break;
 
   case 55:
-#line 402 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 415 "dds.y"
     { (*DDS_OBJ(arg)).set_dataset_name((yyvsp[(1) - (1)].word)); ;}
     break;
 
   case 56:
-#line 404 "dds.y"
+
+/* Line 1455 of yacc.c  */
+#line 417 "dds.y"
     {
 		  ostringstream msg;
 		  msg << "Error parsing the dataset name." << endl
@@ -1862,8 +1945,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1867 "dds.tab.cc"
+
+/* Line 1455 of yacc.c  */
+#line 1951 "dds.tab.cc"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1874,7 +1958,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -1939,7 +2022,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -1956,7 +2039,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2013,9 +2096,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -2040,7 +2120,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2051,7 +2131,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2077,7 +2157,9 @@ yyreturn:
 }
 
 
-#line 414 "dds.y"
+
+/* Line 1675 of yacc.c  */
+#line 427 "dds.y"
 
 
 /* 
