@@ -36,13 +36,15 @@
 #include "config.h"
 
 static char rcsid[] not_used =
-    { "$Id: parser-util.cc 18315 2008-03-03 20:14:44Z jimg $"
+    { "$Id: parser-util.cc 22373 2010-03-23 18:02:57Z jimg $"
     };
 
 #include <cerrno>
 #include <cassert>
 #include <cstring>
 #include <cmath>
+#include <cstdlib>
+
 #include <iostream>
 #include <sstream>
 
@@ -195,7 +197,7 @@ int check_byte(const char *val)
     // especially the case for Java clients where Byte datatypes are
     // signed. 3/20/2000 jhrg
     if ((v < 0 && v < DODS_SCHAR_MIN)
-        || v > 0 && static_cast < unsigned long >(v) > DODS_UCHAR_MAX)
+        || (v > 0 && static_cast < unsigned long >(v) > DODS_UCHAR_MAX))
         return FALSE;
 
     return TRUE;
@@ -239,29 +241,64 @@ int check_uint16(const char *val)
 int check_int32(const char *val)
 {
     char *ptr;
+    errno = 0;
     long v = strtol(val, &ptr, 0);      // `0' --> use val to determine base
 
+
     if ((v == 0 && val == ptr) || *ptr != '\0') {
         return FALSE;
     }
 
-    if (v > DODS_INT_MAX || v < DODS_INT_MIN) {
+    // We need to check errno since strtol return clamps on overflow so the
+    // check against the DODS values below will always pass, even for out of
+    // bounds values in the string. mjohnson 7/20/09
+    if (errno == ERANGE) {
         return FALSE;
     }
-
-    return TRUE;
+    // This could be combined with the above, or course, but I'm making it
+    // separate to highlite the test. On 64-bit linux boxes 'long' may be 
+    // 64-bits and so 'v' can hold more than a DODS_INT32. jhrg 3/23/10
+    else if (v > DODS_INT_MAX || v < DODS_INT_MIN) {
+	return FALSE;
+    }
+    else {
+	return TRUE;
+    }
 }
 
 int check_uint32(const char *val)
 {
+  // Eat whitespace and check for an initial '-' sign...
+  // strtoul allows an initial minus. mjohnson
+    const char* c = val;
+    while (c && isspace(*c)) {
+         c++;
+    }
+    if (c && (*c == '-')) {
+         return FALSE;
+    }
+
     char *ptr;
-    unsigned long v = strtol(val, &ptr, 0);
+    errno = 0;
+    unsigned long v = strtoul(val, &ptr, 0);
 
     if ((v == 0 && val == ptr) || *ptr != '\0') {
         return FALSE;
     }
 
-    return TRUE;
+    // check overflow first, or the below check is invalid due to
+    // clamping to the maximum value by strtoul
+    // maybe consider using long long for these checks? mjohnson
+    if (errno == ERANGE) {
+      return FALSE;
+    }
+    // See above.
+    else if (v > DODS_UINT_MAX) {
+	return FALSE;
+    }
+    else {
+	return TRUE;
+    }
 }
 
 // Check first for system errors (like numbers so small they convert
@@ -272,7 +309,7 @@ int check_float32(const char *val)
 {
     char *ptr;
     errno = 0;                  // Clear previous value. Fix for the 64bit
-    // IRIX from Rob Morris. 5/21/2001 jhrg
+				// IRIX from Rob Morris. 5/21/2001 jhrg
 
 #ifdef WIN32
     double v = w32strtod(val, &ptr);
@@ -282,10 +319,15 @@ int check_float32(const char *val)
 
     DBG(cerr << "v: " << v << ", ptr: " << ptr
         << ", errno: " << errno << ", val==ptr: " << (val == ptr) << endl);
+
+    if (errno == ERANGE || (v == 0.0 && val == ptr) || *ptr != '\0')
+	return FALSE;
+#if 0
     if ((v == 0.0 && (val == ptr || errno == HUGE_VAL || errno == ERANGE))
         || *ptr != '\0') {
         return FALSE;
     }
+#endif
 
     DBG(cerr << "fabs(" << val << ") = " << fabs(v) << endl);
     double abs_val = fabs(v);
@@ -311,11 +353,15 @@ int check_float64(const char *val)
     DBG(cerr << "v: " << v << ", ptr: " << ptr
         << ", errno: " << errno << ", val==ptr: " << (val == ptr) << endl);
 
+
+    if (errno == ERANGE || (v == 0.0 && val == ptr) || *ptr != '\0')
+	return FALSE;
+#if 0
     if ((v == 0.0 && (val == ptr || errno == HUGE_VAL || errno == ERANGE))
         || *ptr != '\0') {
         return FALSE;
     }
-
+#endif
     DBG(cerr << "fabs(" << val << ") = " << fabs(v) << endl);
     double abs_val = fabs(v);
     if (abs_val > DODS_DBL_MAX
