@@ -35,7 +35,7 @@
 
 #include "config.h"
 
-static char rcsid[] not_used = {"$Id: dds-test.cc 19864 2008-11-14 17:01:35Z jimg $"};
+static char rcsid[] not_used = {"$Id: dds-test.cc 22368 2010-03-23 16:51:22Z jimg $"};
 
 #include <GetOpt.h>
 
@@ -102,12 +102,12 @@ main(int argc, char *argv[])
 	    case '?':
 	    default:
 	      usage(argv[0]);
-	      exit(1);
+	      return 1;
 	  }
 
     if (!scanner_test && !parser_test && !class_test) {
 	usage(argv[0]);
-	exit(1);
+	return 1;
     }
 
     try {
