@@ -35,10 +35,10 @@
 
 #include "config.h"
 
-//#define DODS_DEBUG
+#define DODS_DEBUG
 
 static char rcsid[] not_used =
-    { "$Id: expr-test.cc 19917 2008-11-25 23:47:56Z jimg $" };
+    { "$Id: expr-test.cc 22367 2010-03-23 15:36:00Z jimg $" };
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,6 +50,7 @@ static char rcsid[] not_used =
 
 
 #include <iostream>
+#include <fstream>
 #include <string>
 
 #include "GetOpt.h"
@@ -58,11 +59,12 @@ static char rcsid[] not_used =
 #include "DDS.h"
 #include "DataDDS.h"
 #include "ConstraintEvaluator.h"
-#if 0
-#include "XDRFileMarshaller.h"
-#endif
 #include "XDRFileUnMarshaller.h"
+#include "XDRStreamUnMarshaller.h"
 #include "XDRStreamMarshaller.h"
+#include "DODSFilter.h"
+#include "Response.h"
+#include "Connect.h"
 #include "Error.h"
 
 #include "TestSequence.h"
@@ -73,7 +75,9 @@ static char rcsid[] not_used =
 #include "expr.h"
 #include "ce_expr.tab.hh"
 #include "util.h"
-#include "GNU/outbuf.h"
+#if 0
+#include "fdiostream.h"
+#endif
 #include "debug.h"
 
 using namespace std;
@@ -81,7 +85,7 @@ using namespace std;
 int test_variable_sleep_interval = 0;   // Used in Test* classes for testing
                                       // timeouts.
 
-#define CRLF "\r\n"             // Change this here and in cgi_util.cc
+#define CRLF "\r\n"             // Change this here and in mime_util.cc
 #define DODS_DDS_PRX "dods_dds"
 #define YY_BUFFER_STATE (void *)
 
@@ -91,9 +95,6 @@ void test_parser(ConstraintEvaluator & eval, DDS & table,
                  const string & dds_name, const string & constraint);
 bool read_table(DDS & table, const string & name, bool print);
 void evaluate_dds(DDS & table, bool print_constrained);
-#if 0
-bool loopback_pipe(FILE ** pout, FILE ** pin);
-#endif
 void constrained_trans(const string & dds_name, const bool constraint_expr,
                        const string & ce, const bool series_values);
 void intern_data_test(const string & dds_name, const bool constraint_expr,
@@ -244,12 +245,8 @@ int main(int argc, char *argv[])
         }
 
         if (whole_enchalada) {
-#if 1
-            // Broken until we can fix the loopback pipe code to work
-            // with C++ streams
             constrained_trans(dds_file_name, constraint_expr, constraint,
                               series_values);
-#endif
         }
         if (whole_intern_enchalada) {
             intern_data_test(dds_file_name, constraint_expr, constraint,
@@ -417,81 +414,7 @@ void evaluate_dds(DDS & table, bool print_constrained)
             (*p)->print_decl(cout, "", true, true);
 }
 
-// create a pipe for the caller's process which can be used by the DODS
-// software to write to and read from itself.
-#if 0
-bool loopback_pipe(FILE ** pout, FILE ** pin)
-{
-#ifdef WIN32
-    int fd[2];
-    if (_pipe(fd, 1024, _O_BINARY) < 0) {
-        fprintf(stderr, "Could not open pipe\n");
-        return false;
-    }
-
-    *pout = fdopen(fd[1], "w+b");
-    *pin = fdopen(fd[0], "r+b");
-#else
-    int fd[2];
-    if (pipe(fd) < 0) {
-        fprintf(stderr, "Could not open pipe\n");
-        return false;
-    }
-
-    *pout = fdopen(fd[1], "w");
-    *pin = fdopen(fd[0], "r");
-#endif
-
-    return true;
-}
-
-#include "xstream/fd.h"
-
-bool loopback_pipe(ostream pout, istream pin)
-{
-    int fd[2];
-    if (pipe(fd) < 0) {
-        fprintf(stderr, "Could not open pipe\n");
-        return false;
-    }
-
-    xstream::fd::streambuf out(fd[1],true);
-    xstream::fd::streambuf in(fd[0],true);
-
-    pout = std::ostream(&out);
-    pin = std::istream(&in);
-
-    return true;
-}
-#endif
-
-#if 1
-
-#ifndef WIN32
-#define PIPE(x) pipe((x))
-#else
-#define PIPE(x) _pipe((x), 1024, _O_BINARY)
-#endif
-
-#endif
-
-bool
-loopback_pipe(fdostream **pout, FILE **pin)
-{
-    int fd[2];
-    if (pipe(fd) < 0) {
-        fprintf(stderr, "Could not open pipe\n");
-        return false;
-    }
-
-    *pin = fdopen(fd[0], "r");
-    *pout = new fdostream(fd[1]);
-
-    return true;
-}
-
-
-// Gobble up the mime header. At one time the MIME Headers were output from
+// Gobble up the MIME header. At one time the MIME Headers were output from
 // the server filter programs (not the core software) so we could call
 // DDS::send() from this test code and not have to parse the MIME header. But
 // in order to get errors to work more reliably the header generation was
@@ -536,13 +459,6 @@ void
 constrained_trans(const string & dds_name, const bool constraint_expr,
                   const string & constraint, const bool series_values)
 {
-    FILE *pin;
-    fdostream *pout;
-    /* Add error check ***/
-    if (!loopback_pipe(&pout, &pin))
-	throw InternalErr(__FILE__, __LINE__,
-			  "Could not create the loopback pipe.");
-
     // If the CE was not passed in, read it from the command line.
     string ce;
     if (!constraint_expr) {
@@ -572,79 +488,32 @@ constrained_trans(const string & dds_name, const bool constraint_expr,
     // versatility 02/05/07 jhrg
     set_series_values(server, series_values);
 
-    eval.parse_constraint(ce, server);  // Throws Error if the ce doesn't parse.
+    DODSFilter df;
+    df.set_ce(ce);
+    df.set_dataset_name(dds_name);
+    df.set_URL("test://test");
+    df.set_response("DataDDS");
 
-    server.tag_nested_sequences();      // Tag Sequences as Parent or Leaf node.
+    ofstream out("expr-test-data.bin", ios::out|ios::trunc|ios::binary);
+    df.send_data(server, eval, out, "", false);
+    out.close();
 
-    if (eval.functional_expression()) {
-        BaseType *var = eval.eval_function(server, dds_name);
-        if (!var)
-            throw Error(unknown_error, "Error calling the CE function.");
-
-        *pout << "Dataset {\n";
-        var->print_decl(*pout, "    ", true, false, true);
-        *pout << "} function_value;\n";
-        *pout << "Data:\n";
-
-        *pout << flush;
-
-        XDRStreamMarshaller m( *pout ) ;
-
-        try {
-            // In the following call to serialize, suppress CE evaluation.
-            var->serialize(eval, server, m, false);
-        }
-        catch(Error & e) {
-            delete var;
-            var = 0;
-            throw;
-        }
-
-        delete var;
-        var = 0;
-    }
-    else {
-        // send constrained DDS
-        server.print_constrained(*pout);
-        *pout << "Data:\n";
-        *pout << flush;
-
-        // Grab a stream that encodes using XDR.
-        XDRStreamMarshaller m( *pout ) ;
-
-        // Send all variables in the current projection (send_p())
-        for (DDS::Vars_iter i = server.var_begin(); i != server.var_end();
-             i++)
-            if ((*i)->send_p()) {
-                DBG(cerr << "Sending " << (*i)->name() << endl);
-                (*i)->serialize(eval, server, m, true);
-            }
-
-        *pout << flush;
-    }
-
-    delete pout;
     // Now do what Connect::request_data() does:
+    FILE *fp = fopen("expr-test-data.bin", "r");
 
-    // First read the DDS into a new object (using a file to store the DDS
-    // temporarily - the parser/scanner won't stop reading until an EOF is
-    // found, this fixes that problem).
+    Response r(fp, 400);
+    r.set_type(dods_data);
+    r.set_protocol("3.2");
+
+    Connect c("http://dummy_argument");
 
-    // I use the default BaseTypeFactory since we're just printing the
-    // values here.
     BaseTypeFactory factory;
     DataDDS dds(&factory, "Test_data", "DAP/3.1");      // Must use DataDDS on receiving end
-    DBG(cerr << "pin: " << pin << endl);
-    dds.parse(pin);
 
-    XDRFileUnMarshaller um( pin ) ;
-
-    // Back on the client side; deserialize the data *using the newly
-    // generated DDS* (the one sent with the data).
+    c.read_data_no_mime(dds, &r);
 
     cout << "The data:" << endl;
     for (DDS::Vars_iter q = dds.var_begin(); q != dds.var_end(); q++) {
-        (*q)->deserialize(um, &dds);
         (*q)->print_val(cout);
     }
 }
@@ -714,6 +583,13 @@ intern_data_test(const string & dds_name, const bool constraint_expr,
 
     cout << "The data:\n";
 
+    // This code calls 'output_values()' because print_val() does not test
+    // the value of send_p(). We need to wrap a method around the calls to
+    // print_val() to ensure that only values for variables with send_p() set
+    // are called. In the serialize/deserialize case, the 'client' DDS only
+    // has variables sent by the 'server' but in the intern_data() case, the
+    // whole DDS is still present and only variables selected in the CE have
+    // values.
     for (DDS::Vars_iter q = server.var_begin(); q != server.var_end(); q++) {
         if ((*q)->send_p()) {
             (*q)->print_decl(cout, "", false, false, true);
