@@ -96,7 +96,7 @@ class HTTPConnectTest: public TestFixture {
         // above URL. The values below much match the etag and last-modified
         // time returned by the server. Run this test with DODS_DEBUG defined
         // to see the values it's returning.
-        etag = "\"2c004-157-139c2680\"";
+        etag = "\"a10df-157-139c2680\"";
         lm = "Wed, 13 Jul 2005 19:32:26 GMT";
 
         localhost_pw_url =
@@ -104,7 +104,7 @@ class HTTPConnectTest: public TestFixture {
         localhost_digest_pw_url =
             "http://jimg:dods_digest@test.opendap.org/digest/page.txt";
         netcdf_das_url =
-            "http://test.opendap.org/opendap/nph-dods/data/nc/fnoc1.nc.das";
+            "http://test.opendap.org/dap/data/nc/fnoc1.nc.das";
     }
 
     void tearDown() {
@@ -124,6 +124,7 @@ class HTTPConnectTest: public TestFixture {
     CPPUNIT_TEST(set_xdap_protocol_test);
     CPPUNIT_TEST(read_url_password_test);
     CPPUNIT_TEST(read_url_password_test2);
+
 #if 0
     CPPUNIT_TEST(read_url_password_proxy_test);
 #endif
@@ -299,6 +300,7 @@ class HTTPConnectTest: public TestFixture {
     void type_test() {
         Response *r = http->fetch_url(netcdf_das_url);
         try {
+            DBG(cerr << "r->get_type(): " << r->get_type() << endl);
             CPPUNIT_ASSERT(r->get_type() == dods_das);
             delete r;
             r = 0;
